// anti-\nu n -> \mu+ n \pi- channel
#include <cmath>
#include "mkcons.h"
#include "neutmodelC.h"

extern "C" {
  void xsec_im11_(float *diff_Xsec, float *Wf, float *Qf, float *Ef, float *tf, float *phif, float *lepmass);
}

void xsec_im11_(float *diff_Xsec, float *Ef, float *Qf, float *Wf, float *tf, float *phif, float *lepmass) {

  double W = *Wf;
  double Q = *Qf;
  double E = *Ef;
  double t = *tf;
  double phi = *phif;
  double m_l = *lepmass;

  // Get CA50, MA, MV, MAbkg from the NEUT common block
  double C50 = nemdls_.rca5ispi;
  double MAR = nemdls_.xmaspi;
  double M_V = nemdls_.xmvspi;
  double M_A = nemdls_.xmabkgm;

  //kinematic
  double q_0 =(W*W - M*M + m_pi* m_pi)/(2.*W);
  double k_0 = (W*W - M*M - Q)/(2.*W) ;
  double abs_mom_q;
  if (q_0*q_0 - m_pi*m_pi > 0) {
    abs_mom_q = sqrt( q_0*q_0 - m_pi*m_pi); 
  } else {
    abs_mom_q = 0.0;
  }

  double abs_mom_k;
  if (k_0*k_0 + Q > 0) {
    abs_mom_k = sqrt(k_0*k_0 + Q);
  } else {
    abs_mom_k = 0.0;
    *diff_Xsec = float(0.0);
    return;
  }
  double E_2L = (M*M - W*W - Q + 2.*M*E)/(2.*M) ;
  double abs_mom_k_L = (W/M)*abs_mom_k ;
  double k_2 = (M*M + 2.*M*E - W*W -m_l*m_l)/(2.*W) ;
  double k_1 =  k_2 + (W*W - M*M - Q)/(2.*W) ;

  double p_10 = (W*W + M*M + Q)/(2.*W) ;
  double qk = q_0*k_0 - abs_mom_k*abs_mom_q*t ;

  double k_2L;
  if (E_2L*E_2L > m_l*m_l) {
    k_2L = sqrt(E_2L*E_2L - m_l*m_l) ; //magnitude of lepton momentum in lab frame
  } else {
    k_2L = 0;
  }

  double k_2_iso;
  if (k_2*k_2 > m_l*m_l) {
    k_2_iso = sqrt(k_2*k_2 - m_l*m_l) ; //magnitude of lepton momentum in CM frame
  } else {
    k_2_iso = 0;
  }

  ///////////////////////////////////////////////////////////////////////////////////////////

  double cos_theta; 
  if (k_2_iso > 0) {
    cos_theta = (2.*k_1*k_2  - Q - m_l*m_l)/( 2.*k_1*k_2_iso );
    if (cos_theta > 1) {
      cos_theta = 1;
    } else if (cos_theta < -1) {
      cos_theta = -1;
    }
  } else if (2*k_1*k_2-Q-m_l*m_l > 0) {
    cos_theta = 1;
  } else {
    cos_theta = -1;
  }

  double C_L_minus = (abs_mom_k_L/(2.*E))*sqrt(1.-cos_theta )*sqrt( k_1*(k_2 + k_2_iso) )*((k_1 + k_2_iso)/abs_mom_k +1.) ;
  double C_L_plus = -(abs_mom_k_L/(2.*E))*sqrt(1.- cos_theta)*sqrt( k_1*(k_2 + k_2_iso) )*((k_1 + k_2_iso)/abs_mom_k -1.) ;

  double C_R_minus = -(abs_mom_k_L/(2.*E))*sqrt(1.+cos_theta )*sqrt( k_1*(k_2 - k_2_iso) )*((k_1 - k_2_iso)/abs_mom_k -1.) ;
  double C_R_plus = (abs_mom_k_L/(2.*E))*sqrt(1.+ cos_theta)*sqrt( k_1*(k_2 - k_2_iso) )*((k_1 - k_2_iso)/abs_mom_k +1.) ;

  double C_T_minus_square = //equal to u in massless lepton
    C_L_minus*C_L_minus + C_R_minus* C_R_minus;   
  double C_T_plus_square= //equal to v in massless lepton
    C_L_plus *C_L_plus + C_R_plus*C_R_plus ;

  double C_S_plus_square = //equal to zero in massless lepton
    (abs_mom_k_L*abs_mom_k_L/ (2.*E*E))*( k_1*(k_2 - k_2_iso ))*(1. - cos_theta);

  double C_S_minus_square = //equal to sqrt(2.uv) is massless lepton
    (abs_mom_k_L*abs_mom_k_L/ (2.*E*E))*( k_1*(k_2 + k_2_iso )) *(1. + cos_theta) ;



  double eps_zero_L = -1.;
  double eps_zero_R = 1.;

  double eps_z_L =  -( (k_1 - k_2_iso)/abs_mom_k );

  double eps_z_R = ( (k_1 + k_2_iso)/abs_mom_k ) ; 
  ///////////////////////////////////////////////////////////////////////////////////////
  /*    Bkg Contribution   */

  //Auxiliary functions
  double W_plus = W+M ;
  double W_minus = W-M ;
  double O_1_plus = sqrt((W_plus*W_plus + Q)*( W_plus*W_plus - m_pi*m_pi ))/ (2.*W) ;
  double O_1_minus = sqrt((W_minus*W_minus + Q)*(W_minus*W_minus - m_pi*m_pi))/ (2.*W) ;
  double O_2_plus = sqrt((W_plus*W_plus + Q)/(W_plus*W_plus - m_pi*m_pi)) ;
  double O_2_minus = sqrt((W_minus*W_minus + Q)/(W_minus*W_minus - m_pi*m_pi)) ;
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //Vector_helicity amplituds for bkg
  double K_1_V = W_minus*O_1_plus; 
  double K_2_V = W_plus*O_1_minus; 
  double K_3_V = (q_0*q_0 - m_pi*m_pi)*W_plus*O_2_minus; 
  double K_4_V = abs_mom_q*abs_mom_q*W_minus*O_2_plus; 
  double K_5_V = 1./O_2_plus; 
  double K_6_V = 1./O_2_minus; 
  double FF_V ;
  if (W<1.4)
    FF_V = 1. ;//virtual form factor to kill the bkg smothly
  else if (W>=1.4 && W<1.6)
    FF_V =   -23.3077*W*W + 64.9163*W -44.2041   ;
  else 
    FF_V = 0.;

  // Galster vector form-factor
  double mup= 2.792847;
  double mun= -1.913043;
  double Ln= 5.6;
  double tau= Q/(4.*M*M);
  double GEp= 1./((1.+ Q/(M_V*M_V))*(1.+ Q/(M_V*M_V))) ;
  double GEn= -(mun*tau/(1 + Ln*tau))*GEp;
  double GMn= mun*GEp;
  double GMp= mup*GEp;
  double F1n= (GEn + tau*GMn)/(1.+tau);
  double F1p= (GEp + tau*GMp)/(1.+tau);
  double muF2n= (GMn - GEn)/(1.+tau);
  double muF2p= (GMp - GEp)/(1.+tau);

  double FF_1 = 0.5*(F1p - F1n);             

  double FF_2 = 0.5*(muF2p - muF2n);

  double FF_pi = 2.*FF_1 ;

  double V_1 =  (1./sqrt(2.))*(g_A*M/f_pi)*(2.*FF_1*FF_V)*(2./(m_pi*m_pi - 2.*q_0*p_10 - 2.*abs_mom_q*abs_mom_k*t))
    +  (1./sqrt(2.))*(g_A/f_pi)*(FF_2*FF_V/M) ;
  double V_2 =  (1./sqrt(2.))*(g_A*M/f_pi)*(2.*FF_1*FF_V/qk)*(2./(m_pi*m_pi - 2.*q_0*p_10 - 2.*abs_mom_q*abs_mom_k*t)) ;
  double V_3 =  + (1./sqrt(2.))*(g_A*M/f_pi)*(FF_2*FF_V/M)*(2./(m_pi*m_pi - 2.*q_0*p_10 - 2.*abs_mom_q*abs_mom_k*t)) ;
  double V_4 = - (1./sqrt(2.))*(g_A*M/f_pi)*(FF_2*FF_V/M)*(2./(m_pi*m_pi - 2.*q_0*p_10 - 2.*abs_mom_q*abs_mom_k*t)); 

  double V_5 =  -  (1./sqrt(2.))*(g_A*M/f_pi)*(FF_pi*FF_V/qk)*(2./(Q+2.*qk)) ;

  double V_25 = (W_plus*W_minus)*V_2 - Q*V_5; 

  double F_1 = V_1 + qk*(V_3 - V_4)/W_minus + W_minus*V_4; 
  double F_2 = -V_1 + qk*(V_3 - V_4)/W_plus  + W_plus*V_4; 
  double F_3 = (V_3 - V_4) + V_25/W_plus; 
  double F_4 = (V_3 - V_4) - V_25/W_minus; 
  double F_5 = (W_plus*W_plus + Q)*V_1/(2.*W) - qk*(W_plus*W_plus + Q + 2.*W*W_minus)*V_2/(2.*W) + (W_plus*q_0 
      - qk)*(V_3 - V_4) + (W_plus*W_plus + Q)*W_minus*V_4/(2.*W) - k_0*qk*V_5 + q_0*V_25; 

  double F_6 = -(W_minus*W_minus + Q)*V_1/(2.*W) + qk*(W_plus*W_plus + Q + 2.*W*W_minus)*V_2/(2.*W) + (W_minus*q_0 
      - qk)*(V_3 - V_4) + (W_minus*W_minus + Q)*W_plus*V_4/(2.*W) - q_0*V_25  + k_0*qk*V_5; 



  double sF_1 = K_1_V*F_1/(2.*M); 
  double sF_2 = K_2_V*F_2/(2.*M); 
  double sF_3 = K_3_V*F_3/(2.*M); 
  double sF_4 = K_4_V*F_4/(2.*M); 
  double sF_5 = K_5_V*F_5/(2.*M); 
  double sF_6 = K_6_V*F_6/(2.*M); 

  double F_plus11 = -sqrt(2.)*( sqrt((1-t)/2.)*(sF_1 + sF_2) + 0.5*sqrt(1. - t*t)* sqrt((1+t)/2.)*(sF_3 + sF_4) );
  double F_plus_11 = -sqrt(2.)*( sqrt((1+t)/2.)*(sF_1 - sF_2) - 0.5*sqrt(1. - t*t)* sqrt((1-t)/2.)*(sF_3 - sF_4) );
  double F_plus1_1 = (1./sqrt(2.))*sqrt(1. - t*t)* sqrt((1-t)/2.)*(sF_3 - sF_4);
  double F_plus_1_1 = (1./sqrt(2.))*sqrt(1. - t*t)* sqrt((1+t)/2.)*(sF_3 + sF_4);

  double F_minus11 = (1./sqrt(2.))*sqrt(1. - t*t)* sqrt((1+t)/2.)*(sF_3 + sF_4);
  double F_minus_11 = -(1./sqrt(2.))*sqrt(1. - t*t)* sqrt((1-t)/2.)*(sF_3 - sF_4);
  double F_minus1_1 = sqrt(2.)*( sqrt((1+t)/2.)*(sF_1 - sF_2) - 0.5*sqrt(1. - t*t)* sqrt((1-t)/2.)*(sF_3 - sF_4) );
  double F_minus_1_1 = -sqrt(2.)*( sqrt((1-t)/2.)*(sF_1 + sF_2) + 0.5*sqrt(1. - t*t)* sqrt((1+t)/2.)*(sF_3 + sF_4) );

  double F_zero_minus11 = sqrt((1+t)/2.)*(k_0*eps_z_L - abs_mom_k*eps_zero_L)*(sF_5 + sF_6); 
  double F_zero_minus_11 = - sqrt((1-t)/2.)*(k_0*eps_z_L - abs_mom_k*eps_zero_L)*(sF_5 - sF_6); 
  double F_zero_minus1_1 = - sqrt((1-t)/2.)*(k_0*eps_z_L - abs_mom_k*eps_zero_L)*(sF_5 - sF_6);
  double F_zero_minus_1_1 = - sqrt((1+t)/2.)*(k_0*eps_z_L - abs_mom_k*eps_zero_L)*(sF_5 + sF_6);

  double F_zero_plus11 = sqrt((1+t)/2.)*(k_0*eps_z_R - abs_mom_k*eps_zero_R)*(sF_5 + sF_6); 
  double F_zero_plus_11 = - sqrt((1-t)/2.)*(k_0*eps_z_R - abs_mom_k*eps_zero_R)*(sF_5 - sF_6);
  double F_zero_plus1_1 = - sqrt((1-t)/2.)*(k_0*eps_z_R - abs_mom_k*eps_zero_R)*(sF_5 - sF_6);
  double F_zero_plus_1_1 = - sqrt((1+t)/2.)*(k_0*eps_z_R - abs_mom_k*eps_zero_R)*(sF_5 + sF_6);

  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //Axial helicity amp for bkg
  double K_1_A = abs_mom_q*O_2_plus ;
  double K_2_A = abs_mom_q* O_2_minus ;
  double K_3_A = abs_mom_q*O_1_minus ;
  double K_4_A = abs_mom_q* O_1_plus ;
  double K_5_A = O_1_minus ;
  double K_6_A = O_1_plus ;
  // Comment outbecause unused
  //double K_7_A = O_1_minus ;
  //double K_8_A = O_1_plus ;
  double Delta = k_0*(q_0*k_0 - qk)/(abs_mom_k*abs_mom_k) ;

  //Dipole axial form-factor 
  double FF_A =  F0_A/((1+ Q/(M_A*M_A))*(1+ Q/(M_A*M_A))) ;
  double F_rho = F0_rho/(1. + ((Q - m_pi*m_pi + 2.*qk)/(m_rho*m_rho)) ) ;

  double A_1=
    (1./sqrt(2.))*(g_A/f_pi)*FF_A*FF_V*( 2.*M/(m_pi*m_pi - 2.*q_0*p_10 - 2.*abs_mom_q*abs_mom_k*t)) ;

  double A_3 = 
    -(1./sqrt(2.))*(g_A/f_pi)*FF_A*FF_V*( 2.*M/(m_pi*m_pi - 2.*q_0*p_10 - 2.*abs_mom_q*abs_mom_k*t)) ;

  double A_4 = 
    -(1./sqrt(2.))*(g_A/f_pi)*FF_A*FF_V*(1./(M)) + (1./(sqrt(2.)*f_pi))*(1./M)*F_rho*FF_V ;

  double A_7 =
    - (1/sqrt(2.))*(g_A/f_pi)*FF_A*FF_V*(2.*M/(Q + m_pi*m_pi)) ;

  double A_8 =
    (1./sqrt(2.))*(g_A/f_pi)*FF_A*FF_V*( 2./(m_pi*m_pi - 2.*q_0*p_10 - 2.*abs_mom_q*abs_mom_k*t))*(2.*M*M/(Q + m_pi*m_pi)) 
    + (1./sqrt(2.))*(g_A/f_pi)*FF_A*FF_V*(1./(Q + m_pi*m_pi))
    -(1./sqrt(2.))*(1./f_pi)*(1./(Q + m_pi*m_pi))*F_rho*FF_V 
    ;

  double G_1 = W_plus* A_1 - M*A_4 ;
  double G_2 = (-1)*W_minus*A_1 - M*A_4 ;
  double G_3 = A_1 - A_3 ;
  double G_4 = -A_1 + A_3 ;
  double G_5 =
    (Delta + (W_plus*W_plus - m_pi*m_pi)/(2.*W) + 2.*(W*k_0*W_plus)/(W_minus*W_minus + Q))*A_1 
    + (q_0 - Delta)*A_3 - M*W_minus*A_4/(p_10-M);
  double G_6 =
    -(Delta + (W_minus*W_minus - m_pi*m_pi)/(2.*W) + 2.*W*k_0*W_minus/(W_plus*W_plus + Q))*A_1 
    - (q_0 - Delta)*A_3 - M*W_plus*A_4/(p_10 + M);
  double G_7=  ((W_plus*W_plus - m_pi*m_pi)*A_1/(2.*W)) + q_0*A_3 - M*A_4 + k_0*A_7 + W_plus* k_0*A_8 ;
  double G_8 = -((W_minus*W_minus - m_pi*m_pi)*A_1/(2.*W)) - q_0*A_3 - M*A_4 - k_0*A_7 + W_minus* k_0*A_8 ;

  //Isobaric amplitud (Scribal G)

  double sG_1 = K_1_A*G_1/(2.*M) ;
  double sG_2 = K_2_A*G_2/(2.*M) ;
  double sG_3 = K_3_A*G_3/(2.*M) ;
  double sG_4 = K_4_A*G_4/(2.*M) ;
  double sG_5 = K_5_A*G_5/(2.*M) ;
  double sG_6 = K_6_A*G_6/(2.*M) ;
  double sG_7 = K_5_A*G_7/(2.*M) ;
  double sG_8 = K_6_A*G_8/(2.*M) ;

  // helicity amplitudes
  double G_plus11 = -sqrt(2.)*(  sqrt((1-t)/2.)*(sG_1 + sG_2) + 0.5*sqrt(1. - t*t)* sqrt((1+t)/2.)*(sG_3 + sG_4) );
  double G_plus_11 = sqrt(2.)*(  sqrt((1+t)/2.)*(sG_1 - sG_2) - 0.5*sqrt(1. - t*t)* sqrt((1-t)/2.)*(sG_3 - sG_4) );
  double G_plus1_1 = (1./sqrt(2.))*sqrt(1. - t*t)* sqrt((1-t)/2.)*(sG_3 - sG_4);
  double G_plus_1_1 = -(1./sqrt(2.))*sqrt(1. - t*t)* sqrt((1+t)/2.)*(sG_3 + sG_4);

  double G_minus11 =  (1./sqrt(2.))*sqrt(1. - t*t)* sqrt((1+t)/2.)*(sG_3 + sG_4);
  double G_minus_11 =  (1./sqrt(2.))*sqrt(1. - t*t)* sqrt((1-t)/2.)*(sG_3 - sG_4);
  double G_minus1_1 =  sqrt(2.)*(  sqrt((1+t)/2.)*(sG_1 - sG_2) - 0.5*sqrt(1. - t*t)* sqrt((1-t)/2.)*(sG_3 - sG_4) );
  double G_minus_1_1 =  sqrt(2.)*(  sqrt((1-t)/2.)*(sG_1 + sG_2) + 0.5*sqrt(1. - t*t)* sqrt((1+t)/2.)*(sG_3 + sG_4) );


  double G_zero_minus11 = ( sqrt((1+t)/2.)/k_0)*( (abs_mom_k*eps_z_L)*(sG_5 + sG_6) + (k_0*eps_zero_L - abs_mom_k*eps_z_L)*(sG_7 + sG_8) );
  double G_zero_minus_11 = ( sqrt((1-t)/2.)/k_0)*( (abs_mom_k*eps_z_L)*(sG_5 - sG_6) + (k_0*eps_zero_L - abs_mom_k*eps_z_L)*(sG_7 - sG_8) );
  double G_zero_minus1_1 = -( sqrt((1-t)/2.)/k_0)*( (abs_mom_k*eps_z_L)*(sG_5 - sG_6) + (k_0*eps_zero_L - abs_mom_k*eps_z_L)*(sG_7 - sG_8) );
  double G_zero_minus_1_1 = ( sqrt((1+t)/2.)/k_0)*( (abs_mom_k*eps_z_L)*(sG_5 + sG_6) + (k_0*eps_zero_L - abs_mom_k*eps_z_L)*(sG_7 + sG_8) ); 

  double G_zero_plus11 = ( sqrt((1+t)/2.)/k_0)*( (abs_mom_k*eps_z_R)*(sG_5 + sG_6) + (k_0*eps_zero_R - abs_mom_k*eps_z_R)*(sG_7 + sG_8) );
  double G_zero_plus_11 =  ( sqrt((1-t)/2.)/k_0)*( (abs_mom_k*eps_z_R)*(sG_5 - sG_6) + (k_0*eps_zero_R - abs_mom_k*eps_z_R)*(sG_7 - sG_8) );
  double G_zero_plus1_1 = -( sqrt((1-t)/2.)/k_0)*( (abs_mom_k*eps_z_R)*(sG_5 - sG_6) + (k_0*eps_zero_R - abs_mom_k*eps_z_R)*(sG_7 - sG_8) );
  double G_zero_plus_1_1 =  ( sqrt((1+t)/2.)/k_0)*( (abs_mom_k*eps_z_R)*(sG_5 + sG_6) + (k_0*eps_zero_R - abs_mom_k*eps_z_R)*(sG_7 + sG_8) ); 

  ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  /*   Resonace contribution   */

  double N = (W+M)*(W+M) + Q ; 
  double H = M/W ;
  double X = sqrt(2./Omega) ; 
  double L = X*H*abs_mom_k_L ;

  //Form factors

  /*//(1) RS form-factor
    double G_V0 =  (1./((1.+ Q/(M_V*M_V))*(1.+ Q/(M_V*M_V))) )* sqrt(1.+ Q/(4.*M*M)); 
    double G_V1 =  (1./((1.+ Q/(M_V*M_V))*(1.+ Q/(M_V*M_V))) );
    double G_V2 = (1./((1.+ Q/(M_V*M_V))*(1.+ Q/(M_V*M_V))) )* sqrt(1./(1.+ Q/(4.*M*M)));

    double G_A0 = (1./((1.+ Q/( MAR* MAR))*(1.+ Q/( MAR* MAR))) )* sqrt(1.+ Q/(4.*M*M)); 
    double G_A1 = (1./((1.+ Q/( MAR* MAR))*(1.+ Q/( MAR* MAR))) );
    double G_A2 = (1./((1.+ Q/( MAR* MAR))*(1.+ Q/( MAR* MAR))) )* sqrt(1./(1.+ Q/(4.*M*M)));
    */
  //(2) GR Form-factor
  double CV3 = 2.13/( (1.+ Q/(4.*M*M))*(1.+ Q/(M*M))*(1.+ Q/(M*M)) );
  double CV4 = -1.51/( (1.+ Q/(4.*M*M))*(1.+ Q/(M*M))*(1.+ Q/(M*M)) );
  double CV5 = 0.48/( (1.+ Q/(4.*M*M))*(1.+ Q/(0.766*M*M))*(1.+ Q/(0.766*M*M)) );
  double GV3=  (0.5/sqrt(3.))*( CV4*(W*W-Q-M*M)/(2.*M*M) + CV5*(W*W+Q-M*M)/(2.*M*M) +CV3*W_plus/M );
  double GV1= -(0.5/sqrt(3.))*( CV4*(W*W-Q-M*M)/(2.*M*M) + CV5*(W*W+Q-M*M)/(2.*M*M) -CV3*(W_plus*M+Q)/(M*W) );
  double CA5= C50/((1+ Q/(MAR*MAR))*(1+ Q/(MAR*MAR)) );
  double G_V0 = 0.5*sqrt(1.+ Q/(W_plus*W_plus))*sqrt( 3.*GV3*GV3 + GV1*GV1 );
  double G_V1 = 0.5*sqrt(1.+ Q/(W_plus*W_plus))*sqrt( 3.*GV3*GV3 + GV1*GV1 )*sqrt(1./(1.+ Q/(4.*M*M)));
  double G_V2 = 0.5*sqrt(1.+ Q/(W_plus*W_plus))*sqrt( 3.*GV3*GV3 + GV1*GV1 )*(1./(1.+ Q/(4.*M*M)));

  double G_A0 = (1/Z)*0.5*sqrt(3.)*sqrt(1.+ Q/(W_plus*W_plus))*(1 - (W*W -Q -M*M)/(8.*M*M))*CA5;
  double G_A1 = (1/Z)*0.5*sqrt(3.)*sqrt(1.+ Q/(W_plus*W_plus))*(1 - (W*W -Q -M*M)/(8.*M*M))*CA5*sqrt(1./(1.+ Q/(4.*M*M)));
  double G_A2 = (1/Z)*0.5*sqrt(3.)*sqrt(1.+ Q/(W_plus*W_plus))*(1 - (W*W -Q -M*M)/(8.*M*M))*CA5*(1./(1.+ Q/(4.*M*M)));

  double a_aux = 1. + ((W*W + Q + M*M)/(2.*M*W)) ;
  //We multiply C,Band S to abs_mom_k
  double C0_minus = ( (eps_zero_L*abs_mom_k - eps_z_L*k_0)*((1./3.) + k_0/(a_aux*M)) + eps_z_L*((2./3.)*W - Q/(a_aux*M) ) + 
      (eps_zero_L*k_0 - eps_z_L*abs_mom_k)*abs_mom_k*(((2./3.)*W - Q/(a_aux*M))/(m_pi*m_pi + Q)) )*Z*G_A0/(2.*W);
  double C1_minus = ( (eps_zero_L*abs_mom_k - eps_z_L*k_0)*((1./3.) + k_0/(a_aux*M)) + eps_z_L*((2./3.)*W - Q/(a_aux*M) + Omega/(3.*a_aux*M) ) + 
      (eps_zero_L*k_0 - eps_z_L*abs_mom_k)*abs_mom_k*(((2./3.)*W - Q/(a_aux*M) + Omega/(3.*a_aux*M))/(m_pi*m_pi + Q)) )*Z*G_A1/(2.*W);
  double C2_minus = ( (eps_zero_L*abs_mom_k - eps_z_L*k_0)*((1./3.) + k_0/(a_aux*M)) + eps_z_L*((2./3.)*W - Q/(a_aux*M) + 2.*Omega/(3.*a_aux*M) ) + 
      (eps_zero_L*k_0 - eps_z_L*abs_mom_k)*abs_mom_k*(((2./3.)*W - Q/(a_aux*M) + 2.*Omega/(3.*a_aux*M))/(m_pi*m_pi + Q)) )*Z*G_A2/(2.*W);

  double C0_plus = ( (eps_zero_R*abs_mom_k - eps_z_R*k_0)*((1./3.) + k_0/(a_aux*M)) + eps_z_R*((2./3.)*W - Q/(a_aux*M) ) +   
      (eps_zero_R*k_0 - eps_z_R*abs_mom_k)*abs_mom_k*(((2./3.)*W - Q/(a_aux*M))/(m_pi*m_pi + Q)) )*Z*G_A0/(2.*W) ;
  double C1_plus = ( (eps_zero_R*abs_mom_k - eps_z_R*k_0)*((1./3.) + k_0/(a_aux*M)) + eps_z_R*((2./3.)*W - Q/(a_aux*M) + Omega/(3.*a_aux*M) ) +   
      (eps_zero_R*k_0 - eps_z_R*abs_mom_k)*abs_mom_k*(((2./3.)*W - Q/(a_aux*M) + Omega/(3.*a_aux*M))/(m_pi*m_pi + Q)) )*Z*G_A1/(2.*W) ;                     
  double C2_plus =  ( (eps_zero_R*abs_mom_k - eps_z_R*k_0)*((1./3.) + k_0/(a_aux*M)) + eps_z_R*((2./3.)*W - Q/(a_aux*M) + 2.*Omega/(3.*a_aux*M) ) +   
      (eps_zero_R*k_0 - eps_z_R*abs_mom_k)*abs_mom_k*(((2./3.)*W - Q/(a_aux*M) + 2.*Omega/(3.*a_aux*M))/(m_pi*m_pi + Q)) )*Z*G_A2/(2.*W) ;                     

  double B1_minus = sqrt(Omega/2)* (eps_zero_L + eps_z_L*abs_mom_k/(a_aux*M))*Z*G_A1/(3.*W)+ 
    sqrt(Omega/2)* (eps_zero_L*k_0 - eps_z_L*abs_mom_k)* ((k_0 + abs_mom_k*abs_mom_k/(2.*M*a_aux))/(m_pi*m_pi + Q)) *Z*G_A1/(3.*W) ;
  double B2_minus = sqrt(Omega/2)* (eps_zero_L + eps_z_L*abs_mom_k/(a_aux*M))*Z*G_A2/(3.*W)+ 
    sqrt(Omega/2)* (eps_zero_L*k_0 - eps_z_L*abs_mom_k)* ((k_0 + abs_mom_k*abs_mom_k/(2.*M*a_aux))/(m_pi*m_pi + Q)) *Z*G_A2/(3.*W) ;

  double B1_plus = sqrt(Omega/2)* (eps_zero_R + eps_z_R*abs_mom_k/(a_aux*M))*Z*G_A1/(3.*W)+ 
    sqrt(Omega/2)* (eps_zero_R*k_0 - eps_z_R*abs_mom_k)* ((k_0 + abs_mom_k*abs_mom_k/(2.*M*a_aux))/(m_pi*m_pi + Q)) *Z*G_A1/(3.*W) ;
  double B2_plus = sqrt(Omega/2)* (eps_zero_R + eps_z_R*abs_mom_k/(a_aux*M))*Z*G_A2/(3.*W)+ 
    sqrt(Omega/2)* (eps_zero_R*k_0 - eps_z_R*abs_mom_k)* ((k_0 + abs_mom_k*abs_mom_k/(2.*M*a_aux))/(m_pi*m_pi + Q)) *Z*G_A2/(3.*W) ;

  double S1_KLM_minus = abs_mom_k*(eps_z_L*k_0 - eps_zero_L*abs_mom_k)* (1. + Q/(M*M) - 3.*W/M)*( G_V1/(6.*abs_mom_k_L* abs_mom_k_L)) ;
  double S2_KLM_minus = abs_mom_k*(eps_z_L*k_0 - eps_zero_L*abs_mom_k)* (1. + Q/(M*M) - 3.*W/M)*( G_V2/(6.*abs_mom_k_L* abs_mom_k_L)) ;

  double S1_KLM_plus = abs_mom_k*(eps_z_R*k_0 - eps_zero_R*abs_mom_k)* (1. + Q/(M*M) - 3.*W/M)*( G_V1/(6.*abs_mom_k_L* abs_mom_k_L))  ;
  double S2_KLM_plus = abs_mom_k*(eps_z_R*k_0 - eps_zero_R*abs_mom_k)* (1. + Q/(M*M) - 3.*W/M)*( G_V2/(6.*abs_mom_k_L* abs_mom_k_L))  ;

  double R0_V = sqrt(2.)*H*( (W+M)*abs_mom_k_L/N )* G_V0 ;
  double R1_V = sqrt(2.)*H*( (W+M)*abs_mom_k_L/N )* G_V1 ;
  double R2_V = sqrt(2.)*H*( (W+M)*abs_mom_k_L/N )* G_V2 ;

  double R0_A = (sqrt(2.)*Z/(6.*W))*(W + M)* G_A0 ;
  double R1_A = (sqrt(2.)*Z/(6.*W))*(W + M + 2.*Omega*W/N)* G_A1 ;
  double R2_A = (sqrt(2.)*Z/(6.*W))*(W + M + 4.*Omega*W/N)* G_A2 ;

  // Comment outbecause unused
  //double T0_V = G_V0/(3.*W*X) ;
  double T1_V = G_V1/(3.*W*X) ;
  double T2_V = G_V2/(3.*W*X) ;

  // Comment outbecause unused
  //double T0_A = (2./3.)*(Z * H* abs_mom_k_L/(N*X))*G_A0 ;
  double T1_A = (2./3.)*(Z * H* abs_mom_k_L/(N*X))*G_A1 ;
  double T2_A = (2./3.)*(Z * H* abs_mom_k_L/(N*X))*G_A2 ;

  double R0_plus = -(R0_V + R0_A ) ;
  double R1_plus = -(R1_V + R1_A ) ;
  double R2_plus = -(R2_V + R2_A ) ;

  double R0_minus = -(R0_V - R0_A ) ;
  double R1_minus = -(R1_V - R1_A ) ;
  double R2_minus = -(R2_V - R2_A ) ;

  double T1_plus = -(T1_V + T1_A ) ;
  double T2_plus = -(T2_V + T2_A ) ;

  double T1_minus = -(T1_V - T1_A ) ;
  double T2_minus = -(T2_V - T2_A ) ;

  //kapa is define in eq:29 of Rein's paper and the defined kapa below should be multiplied to sqrt(2/2j+1), but we multiply it to amplitudes of each resonances
  ////////////////////////////////////////////////////////////////////////////////////
  //We multiply each resonances to sqrt(2/2j+1) for kapa and multiply to (2j+1)/2 from resonance helicity amplitudes (will be define in my paper)
  //////////////////////////////////////////////////////////////////////
  double  HA_realM11[17],HA_realM_11[17], HA_realM1_1[17], HA_realM_1_1[17] ;    
  double  HA_realP11[17],HA_realP_11[17], HA_realP1_1[17], HA_realP_1_1[17] ;   
  double  HA0_realM11[17],HA0_realM_11[17], HA0_realM1_1[17], HA0_realM_1_1[17] ;    
  double  HA0_realP11[17],HA0_realP_11[17], HA0_realP1_1[17], HA0_realP_1_1[17] ;    

  double  HA_ImM11[17],HA_ImM_11[17], HA_ImM1_1[17], HA_ImM_1_1[17] ;    
  double  HA_ImP11[17],HA_ImP_11[17], HA_ImP1_1[17], HA_ImP_1_1[17] ;    
  double  HA0_ImM11[17],HA0_ImM_11[17], HA0_ImM1_1[17], HA0_ImM_1_1[17] ;    
  double  HA0_ImP11[17],HA0_ImP_11[17], HA0_ImP1_1[17], HA0_ImP_1_1[17] ; 

  double qMR_0[17], abs_mom_qMR[17], Gamma[17], f_BW_real[17], f_BW_Im[17];
  double f1, f_1, f3, f_3, f0_minus,f0_plus, f0R_minus,f0R_plus;
  double kapa[17];
  for (int i=0; i<17; i++){
    qMR_0[i] =(MR[i]*MR[i] - M*M + m_pi* m_pi)/(2.*MR[i]);
    abs_mom_qMR[i] = sqrt( qMR_0[i]*qMR_0[i] - m_pi*m_pi);
    kapa[i] = (pi*W/M)*sqrt(2./(JP[i]*abs_mom_q)) ;
    Gamma[i] = RWA[i]*pow((abs_mom_q/ abs_mom_qMR[i]), LP[i]);
    f_BW_real[i] = NA[i]*sqrt(BRA[i]*Gamma[i]/(2.*pi))*((W-MR[i])/( (W-MR[i])*(W-MR[i]) + Gamma[i]*Gamma[i]/4.) ) ;
    f_BW_Im[i] =  -NA[i]*sqrt(BRA[i]*Gamma[i]/(2.*pi))*((Gamma[i]/2.)/( (W-MR[i])*(W-MR[i]) + Gamma[i]*Gamma[i]/4.) ) ;
    // Correction Minoo Sep 2017
    //f_BW_real[i] = -NA[i]*sqrt(BRA[i]*Gamma[i]/(2.*pi))*((W-MR[i])/( (W-MR[i])*(W-MR[i]) + Gamma[i]*Gamma[i]/4.) ) ;
    //f_BW_Im[i] =  NA[i]*sqrt(BRA[i]*Gamma[i]/(2.*pi))*((Gamma[i]/2.)/( (W-MR[i])*(W-MR[i]) + Gamma[i]*Gamma[i]/4.) ) ;
    if (i==0){
      //P_{33}(1232) **** IBLOCK=0  ****
      f_3 = sqrt(6.)* R0_minus ;
      f_1 = sqrt(2.)* R0_minus ;
      f1 = -sqrt(2.)*R0_plus ;
      f3 = -sqrt(6.)*R0_plus ; 
      f0_plus = -2.*sqrt(2.)*C0_minus ;
      f0_minus = -2.*sqrt(2.)*C0_minus ;
      f0R_plus = -2.*sqrt(2.)*C0_plus ;
      f0R_minus = -2.*sqrt(2.)*C0_plus ;
    }else if (i==1){
      //p_{11}(1440) **** IBLOCK=1  **** 
      f_3 = 0.;
      f_1 =  -(5./6.)*sqrt(3.)*L*L*R2_minus ;
      f1 =   -(5./6.)*sqrt(3.)*L*L*R2_plus ;
      f3 = 0.;
      f0_plus =  -sqrt(3./4.)*L*L*S2_KLM_minus - (5./6.)*sqrt(3.)*L*( L*C2_minus - 2.*B2_minus );
      f0_minus = -sqrt(3./4.)*L*L*S2_KLM_minus + (5./6.)*sqrt(3.)*L*( L*C2_minus - 2.*B2_minus ); 
      f0R_plus = -sqrt(3./4.)*L*L*S2_KLM_plus - (5./6.)*sqrt(3.)*L*( L*C2_plus - 2.*B2_plus );
      f0R_minus= -sqrt(3./4.)*L*L*S2_KLM_plus + (5./6.)*sqrt(3.)*L*( L*C2_plus - 2.*B2_plus ); 
    }else if (i==2){
      //D_{13}(1520)   **** IBLOCK=2  **** 
      f_3 = 2.*sqrt(9./2.)* T1_minus ;
      f_1 =    sqrt(6.)* T1_minus - (4./sqrt(3.))*L*R1_minus ;
      f1  =    sqrt(6.)* T1_plus - (4./sqrt(3.))*L*R1_plus ;
      f3  = 2.*sqrt(9./2.)* T1_plus ;
      f0_plus = -2.*sqrt(3.)*L*S1_KLM_minus - (4./sqrt(3.))*L*C1_minus ;
      f0_minus= -2.*sqrt(3.)*L*S1_KLM_minus + (4./sqrt(3.))*L*C1_minus ;
      f0R_plus= -2.*sqrt(3.)*L*S1_KLM_plus  - (4./sqrt(3.))*L*C1_plus ;
      f0R_minus=-2.*sqrt(3.)*L*S1_KLM_plus  + (4./sqrt(3.))*L*C1_plus ;
    }else if (i==3){
      //S_{11}(1535)   **** IBLOCK=3  **** 
      f_3 =  0.;
      f_1 =  2.*sqrt(3.)*T1_minus + (4./sqrt(6.))*L*R1_minus ;
      f1  = -2.*sqrt(3.)*T1_plus - (4./sqrt(6.))*L*R1_plus ;
      f3  =  0.;
      f0_plus  =  sqrt(6.)*L*S1_KLM_minus + 2.*sqrt(2./3.)*( L*C1_minus - 3.*B1_minus ) ;
      f0_minus = -sqrt(6.)*L*S1_KLM_minus + 2.*sqrt(2./3.)*( L*C1_minus - 3.*B1_minus ) ;
      f0R_plus =  sqrt(6.)*L*S1_KLM_plus  + 2.*sqrt(2./3.)*( L*C1_plus - 3.*B1_plus ) ;
      f0R_minus= -sqrt(6.)*L*S1_KLM_plus  + 2.*sqrt(2./3.)*( L*C1_plus - 3.*B1_plus ) ;
    }else if (i==4){
      //S_{31}(1620)    **** IBLOCK=4  **** 
      f_3 =  0.;
      f_1 = -sqrt(3.)* T1_minus + sqrt(1./6.)*L*R1_minus ;
      f1  =  sqrt(3.)* T1_plus - sqrt(1./6.)*L*R1_plus ;
      f3  =  0.;
      f0_plus  = -sqrt(3./2.)*L*S1_KLM_minus + sqrt(1./6.)*( L*C1_minus - 3.*B1_minus ) ;
      f0_minus =  sqrt(3./2.)*L*S1_KLM_minus + sqrt(1./6.)*( L*C1_minus - 3.*B1_minus ) ;
      f0R_plus = -sqrt(3./2.)*L*S1_KLM_plus  + sqrt(1./6.)*( L*C1_plus - 3.*B1_plus ) ;
      f0R_minus=  sqrt(3./2.)*L*S1_KLM_plus  + sqrt(1./6.)*( L*C1_plus - 3.*B1_plus ) ;
    }else if (i==5){
      //S_{11}(1650)  **** IBLOCK=5  **** 
      f_3 =  0.;
      f_1 =  sqrt(1./6.)*L*R1_minus ;
      f1  = -sqrt(1./6.)*L*R1_plus ;
      f3  =  0.;
      f0_plus  = -sqrt(2./3.)*( L*C1_minus- 3.*B1_minus ) ;
      f0_minus = -sqrt(2./3.)*( L*C1_minus- 3.*B1_minus ) ;
      f0R_plus = -sqrt(2./3.)*( L*C1_plus - 3.*B1_plus ) ;
      f0R_minus= -sqrt(2./3.)*( L*C1_plus - 3.*B1_plus ) ;
    }else if (i==6){
      //D_{15}(1675)  **** IBLOCK=6  **** 
      f_3 = -sqrt(3./5.)*L* R1_minus ;
      f_1 = -sqrt(3./10.)*L* R1_minus ;
      f1  =  sqrt(3./10.)*L* R1_plus ; 
      f3  =  sqrt(3./5.)*L* R1_plus ; 
      f0_plus  = sqrt(6./5.)*L*C1_minus ;
      f0_minus = sqrt(6./5.)*L*C1_minus ;
      f0R_plus = sqrt(6./5.)*L*C1_plus ;
      f0R_minus= sqrt(6./5.)*L*C1_plus ;
    }else if (i==7){
      //F_{15(1680)  **** IBLOCK=7  **** 
      f_3 =  -sqrt(18./5.)*L*T2_minus ;
      f_1 =  -sqrt(9./5.)*L*T2_minus + sqrt(5./2.)*L*L* R2_minus ;
      f1  =  -sqrt(9./5.)*L*T2_plus  + sqrt(5./2.)*L*L* R2_plus ; 
      f3  =  -sqrt(18./5.)*L*T2_plus ;
      f0_plus  = sqrt(9./10.)*L*L*S2_KLM_minus + sqrt(5./2.)*L*L*C2_minus ;
      f0_minus = sqrt(9./10.)*L*L*S2_KLM_minus - sqrt(5./2.)*L*L*C2_minus ;
      f0R_plus = sqrt(9./10.)*L*L*S2_KLM_plus  + sqrt(5./2.)*L*L*C2_plus ;
      f0R_minus= sqrt(9./10.)*L*L*S2_KLM_plus  - sqrt(5./2.)*L*L*C2_plus ;
    }else if (i==8){
      //D_{13}(1700) **** IBLOCK=8  **** 
      f_3 = sqrt(9./10.)*L* R1_minus ;
      f_1 = sqrt(1./30.)*L* R1_minus ;
      f1 =  sqrt(1./30.)*L* R1_plus ;
      f3 =  sqrt(9./10.)*L* R1_plus ;
      f0_plus  = -sqrt(2./15.)*L*C1_minus ;
      f0_minus =  sqrt(2./15.)*L*C1_minus ;
      f0R_plus = -sqrt(2./15.)*L*C1_plus ;
      f0R_minus=  sqrt(2./15.)*L*C1_plus ;
    }else if (i==9){
      //D_{33}(1700)  **** IBLOCK=9  **** 
      f_3 = -sqrt(9./2.)* T1_minus ;
      f_1 = -sqrt(3./2.)* T1_minus - sqrt(1./3.)*L* R1_minus ;
      f1  = -sqrt(3./2.)* T1_plus  - sqrt(1./3.)*L* R1_plus ;
      f3  = -sqrt(9./2.)* T1_plus ;
      f0_plus  = sqrt(3.)*L* S1_KLM_minus - sqrt(1./3.)*L*C1_minus ;
      f0_minus = sqrt(3.)*L* S1_KLM_minus + sqrt(1./3.)*L*C1_minus ;
      f0R_plus = sqrt(3.)*L* S1_KLM_plus  - sqrt(1./3.)*L*C1_plus ;
      f0R_minus= sqrt(3.)*L* S1_KLM_plus  + sqrt(1./3.)*L*C1_plus ;
    }else if (i==10){
      //P_{11}(1710)    **** IBLOCK=10  ****
      f_3 = 0.;
      f_1 = sqrt(2./3.)*L*L*R2_minus ;
      f1 =  sqrt(2./3.)*L*L*R2_plus ;
      f3 = 0.;
      f0_plus  = sqrt(3./2.)*L*L*S2_KLM_minus + sqrt(2./3.)*L*( L*C2_minus - 2.*B2_minus ) ;
      f0_minus = sqrt(3./2.)*L*L*S2_KLM_minus - sqrt(2./3.)*L*( L*C2_minus - 2.*B2_minus ) ;
      f0R_plus = sqrt(3./2.)*L*L*S2_KLM_plus  + sqrt(2./3.)*L*( L*C2_plus - 2.*B2_plus ) ;
      f0R_minus= sqrt(3./2.)*L*L*S2_KLM_plus  - sqrt(2./3.)*L*( L*C2_plus - 2.*B2_plus ) ;
    }else if (i==11){
      //P_{13}(1720)   **** IBLOCK=11  ****
      f_3 =  sqrt(9./10.)*L*T2_minus ;
      f_1 = -sqrt(27./10.)*L*T2_minus - sqrt(5./3.)*L*L* R2_minus  ;
      f1  =  sqrt(27./10.)*L*T2_plus  + sqrt(5./3.)*L*L* R2_plus  ; 
      f3  = -sqrt(9./10.)*L*T2_plus ;
      f0_plus  = -sqrt(3./5.)*L*L*S2_KLM_minus - sqrt(5./3.)*L*(L*C2_minus - 5.*B2_minus )  ;
      f0_minus =  sqrt(3./5.)*L*L*S2_KLM_minus - sqrt(5./3.)*L*(L*C2_minus - 5.*B2_minus )  ; 
      f0R_plus = -sqrt(3./5.)*L*L*S2_KLM_plus  - sqrt(5./3.)*L*(L*C2_plus - 5.*B2_plus ) ;
      f0R_minus=  sqrt(3./5.)*L*L*S2_KLM_plus  - sqrt(5./3.)*L*(L*C2_plus - 5.*B2_plus ) ;
    }else if (i==12){
      //F_{35}(1905)    **** IBLOCK=12  ****
      f_3 = -sqrt(18./35.)*L*L* R2_minus ;
      f_1 = -sqrt(1./35.)*L*L* R2_minus ;
      f1  = -sqrt(1./35.)*L*L* R2_plus ;
      f3  = -sqrt(18./35.)*L*L* R2_plus ;
      f0_plus  =  sqrt(4./35.)*L*L*C2_minus ;
      f0_minus = -sqrt(4./35.)*L*L*C2_minus ;
      f0R_plus =  sqrt(4./35.)*L*L*C2_plus ;
      f0R_minus= -sqrt(4./35.)*L*L*C2_plus ;
    }else if (i==13){
      //P_{31}(1910)    **** IBLOCK=13  ****
      f_3 = 0.;
      f_1 = sqrt(1./15.)*L*L*R2_minus ;
      f1  = sqrt(1./15.)*L*L*R2_plus ;
      f3  = 0.;
      f0_plus  = -sqrt(4./15.)*L*( L*C2_minus - 5.*B2_minus );
      f0_minus =  sqrt(4./15.)*L*( L*C2_minus - 5.*B2_minus );
      f0R_plus = -sqrt(4./15.)*L*( L*C2_plus  - 5.*B2_plus );
      f0R_minus=  sqrt(4./15.)*L*( L*C2_plus  - 5.*B2_plus );
    }else if (i==14){
      //P_{33}(1920)   **** IBLOCK=14  ****
      f_3 =  sqrt(1./5.)*L*L* R2_minus ;
      f_1 = -sqrt(1./15.)*L*L* R2_minus ;
      f1  =  sqrt(1./15)*L*L* R2_plus ;
      f3  = -sqrt(1./5.)*L*L* R2_plus ;
      f0_plus  = sqrt(4./15.)*L*( L*C2_minus - 5.*B2_minus );
      f0_minus = sqrt(4./15.)*L*( L*C2_minus - 5.*B2_minus );
      f0R_plus = sqrt(4./15.)*L*( L*C2_plus  - 5.*B2_plus );
      f0R_minus= sqrt(4./15.)*L*( L*C2_plus  - 5.*B2_plus );
    }else if (i==15){
      //F_{37}(1950)   **** IBLOCK=15  ****
      f_3 =  sqrt(2./7.)*L*L* R2_minus ;
      f_1 =  sqrt(6./35.)*L*L* R2_minus ;
      f1  = -sqrt(6./35.)*L*L* R2_plus ;
      f3  = -sqrt(2./7.)*L*L* R2_plus ;
      f0_plus  = -2.*sqrt(6./35.)*L*L*C2_minus ;
      f0_minus = -2.*sqrt(6./35.)*L*L*C2_minus ;
      f0R_plus = -2.*sqrt(6./35.)*L*L*C2_plus ;
      f0R_minus= -2.*sqrt(6./35.)*L*L*C2_plus ;
    }else if (i==16){
      //P_{33}(1600)   **** IBLOCK=16  ****
      f_3 = -sqrt(1./2.)*L*L*R2_minus ;
      f_1 = -sqrt(1./6.)*L*L*R2_minus ;
      f1  = sqrt(1./6.)*L*L*R2_plus ;
      f3  = sqrt(1./2.)*L*L*R2_plus ;
      f0_plus  = sqrt(2./3.)*L*(L*C2_minus-2.*B2_minus) ;
      f0_minus = f0_plus ;
      f0R_plus = sqrt(2./3.)*L*(L*C2_plus-2.*B2_plus) ;
      f0R_minus= f0R_plus ;

    }
    HA_realM11[i]  = -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f3*Jsgn[i]  ;
    HA_realM_11[i] = -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f3;
    HA_realM1_1[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f1*Jsgn[i] ;
    HA_realM_1_1[i]= -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f1 ;
     
    HA_realP11[i]  =   sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f_1 ;
    HA_realP_11[i] =  -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f_1*Jsgn[i];
    HA_realP1_1[i] =  -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f_3;
    HA_realP_1_1[i]=   sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f_3*Jsgn[i] ;
//abs_mom_k is already multiplied to B, C and S_KLM. 1/sqrt(Q) is cancelled with Q in zero polarization part of Xsec formula
    HA0_realM11[i]  = -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0_minus*Jsgn[i] ;
    HA0_realM_11[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0_minus;
    HA0_realM1_1[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0_plus*Jsgn[i] ;
    HA0_realM_1_1[i]= -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0_plus ;
    
    HA0_realP11[i]  = -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0R_minus*Jsgn[i] ;
    HA0_realP_11[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0R_minus;
    HA0_realP1_1[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0R_plus*Jsgn[i]  ;
    HA0_realP_1_1[i]= -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0R_plus ;
  
    HA_ImM11[i]  =    -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f3*Jsgn[i] ;
    HA_ImM_11[i] =    -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f3;
    HA_ImM1_1[i] =     sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f1*Jsgn[i] ;
    HA_ImM_1_1[i]=    -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f1 ;

    HA_ImP11[i]  =   sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f_1 ;
    HA_ImP_11[i] =  -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f_1*Jsgn[i];
    HA_ImP1_1[i] =  -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f_3 ;
    HA_ImP_1_1[i]=   sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f_3*Jsgn[i] ;

    HA0_ImM11[i]  = -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0_minus*Jsgn[i] ;
    HA0_ImM_11[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0_minus;
    HA0_ImM1_1[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0_plus*Jsgn[i]  ;
    HA0_ImM_1_1[i]= -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0_plus ;

    HA0_ImP11[i]  = -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0R_minus*Jsgn[i] ;
    HA0_ImP_11[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0R_minus;
    HA0_ImP1_1[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0R_plus*Jsgn[i]  ;
    HA0_ImP_1_1[i]= -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0R_plus ; 
    // Correction Minoo Sep 2017
    /*
    HA_realM11[i]  =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f3*Cjsgn_minus[i]  ;
    HA_realM_11[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f3*Cjsgn_plus[i];
    HA_realM1_1[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f1*Cjsgn_minus[i] ;
    HA_realM_1_1[i]=  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f1*Cjsgn_plus[i] ;

    HA_realP11[i]  =  -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f_1*Cjsgn_minus[i] ;
    HA_realP_11[i] =   sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f_1*Cjsgn_plus[i];
    HA_realP1_1[i] =   sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f_3*Cjsgn_minus[i];
    HA_realP_1_1[i]=  -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f_3*Cjsgn_plus[i] ;
    //abs_mom_k is already multiplied to B, C and S_KLM. 1/sqrt(Q) is cancelled with Q in zero polarization part of Xsec formula
    HA0_realM11[i]  =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0_minus*Cjsgn_minus[i] ;
    HA0_realM_11[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0_minus*Cjsgn_plus[i];
    HA0_realM1_1[i] = -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0_plus*Cjsgn_minus[i] ;
    HA0_realM_1_1[i]=  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0_plus*Cjsgn_plus[i] ;

    HA0_realP11[i]  =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0R_minus*Cjsgn_minus[i] ;
    HA0_realP_11[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0R_minus*Cjsgn_plus[i];
    HA0_realP1_1[i] = -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0R_plus *Cjsgn_minus[i] ;
    HA0_realP_1_1[i]=  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_real[i]*f0R_plus *Cjsgn_plus[i];

    HA_ImM11[i]  =     sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f3*Cjsgn_minus[i] ;
    HA_ImM_11[i] =     sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f3*Cjsgn_plus[i];
    HA_ImM1_1[i] =     sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f1*Cjsgn_minus[i] ;
    HA_ImM_1_1[i]=     sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f1*Cjsgn_plus[i] ;

    HA_ImP11[i]  =  -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f_1*Cjsgn_minus[i] ;
    HA_ImP_11[i] =   sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f_1*Cjsgn_plus[i];
    HA_ImP1_1[i] =   sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f_3*Cjsgn_minus[i] ;
    HA_ImP_1_1[i]=  -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f_3*Cjsgn_plus[i] ;

    HA0_ImM11[i]  =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0_minus*Cjsgn_minus[i] ;
    HA0_ImM_11[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0_minus*Cjsgn_plus[i];
    HA0_ImM1_1[i] = -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0_plus *Cjsgn_minus[i] ;
    HA0_ImM_1_1[i]=  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0_plus*Cjsgn_plus[i] ;

    HA0_ImP11[i]  =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0R_minus*Cjsgn_minus[i] ;
    HA0_ImP_11[i] =  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0R_minus*Cjsgn_plus[i];
    HA0_ImP1_1[i] = -sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0R_plus *Cjsgn_minus[i] ;
    HA0_ImP_1_1[i]=  sqrt(2.)*JP[i]*Dsgn[i]*kapa[i]*f_BW_Im[i]*f0R_plus*Cjsgn_plus[i] ;
    */



    }

    double pt_1 = 1. ;
    double pt_2 = 3.*t ;
    double pt_3 = (15./2.)*t*t -3./2. ;
    double pt_4 = (35./2.)*t*t*t - (15./2.)*t ;

    double d111 = sqrt((1 + t)/2.);
    double d11_1 =  -sqrt((1- t)/2.);
    double d131 = 0.;
    double d13_1 = 0.;

    double d311 = (1./2.)*sqrt((1 + t)/2.)*(pt_2 - pt_1) ;
    double d31_1 =(-1./2.)*sqrt((1- t)/2.)*(pt_2 + pt_1) ;
    double d331 =(-1./2.)*sqrt((1- t)/2.)*(sqrt(1./3.)*pt_2 + sqrt(3.)*pt_1) ;
    double d33_1 =(1./2.)*sqrt((1 + t)/2.)*(-sqrt(1./3.)*pt_2 + sqrt(3.)*pt_1) ;

    double d511 = (1./3.)* sqrt((1+t)/2.)*(pt_3 - pt_2) ;
    double d51_1 = (-1./3.)* sqrt((1-t)/2.)*(pt_3 + pt_2);
    double d531 = (-1./3.)* sqrt((1-t)/2.)*( sqrt(1./2.)*pt_3 + sqrt(2.)*pt_2); 
    double d53_1 = (1./3.)* sqrt((1+t)/2.)*(-sqrt(1./2.)*pt_3 + sqrt(2.)*pt_2); 


    double d711 = (1./4.)* sqrt((1+t)/2.)*(pt_4 - pt_3);
    double d71_1 = (-1./4.)* sqrt((1-t)/2.)*(pt_4 + pt_3);
    double d731 = (-1./4.)* sqrt((1-t)/2.)*(sqrt(3./5.)*pt_4 + sqrt(5./3.)*pt_3);
    double d73_1 = (1./4.)* sqrt((1+t)/2.)*(-sqrt(3./5.)*pt_4 + sqrt(5./3.)*pt_3) ;

    double sum3Re_M11  = HA_realM11[0]*  d331 + HA_realM11[4]* d131 + HA_realM11[9]*  d331 + HA_realM11[12]* d531 + HA_realM11[13]* d131 + HA_realM11[14]* d331 + HA_realM11[15]*d731 + HA_realM11[16]* d331;
    double sum3Re_M_11 = HA_realM_11[0]* d33_1+ HA_realM_11[4]*d13_1+ HA_realM_11[9]*d33_1 + HA_realM_11[12]*d53_1 +HA_realM_11[13]*d13_1+ HA_realM_11[14]*d33_1 + HA_realM_11[15]*d73_1 + HA_realM_11[16]*d33_1;
    double sum3Re_M1_1 = HA_realM1_1[0]* d311 + HA_realM1_1[4]*d111 + HA_realM1_1[9]* d311 + HA_realM1_1[12]*d511 + HA_realM1_1[13]*d111 + HA_realM1_1[14]* d311 + HA_realM1_1[15]*d711 + HA_realM1_1[16]* d311;
    double sum3Re_M_1_1= HA_realM_1_1[0]*d31_1+ HA_realM_1_1[4]*d11_1+HA_realM_1_1[9]*d31_1+ HA_realM_1_1[12]*d51_1+HA_realM_1_1[13]*d11_1+ HA_realM_1_1[14]*d31_1+HA_realM_1_1[15]*d71_1+ HA_realM_1_1[16]*d31_1;

    double sum3Re_P11  = HA_realP11[0]*d31_1 + HA_realP11[4]*d11_1 + HA_realP11[9]*d31_1 + HA_realP11[12]*d51_1 + HA_realP11[13]*d11_1 + HA_realP11[14]*d31_1 + HA_realP11[15]*d71_1 + HA_realP11[16]*d31_1;
    double sum3Re_P_11 = HA_realP_11[0]*d311 + HA_realP_11[4]*d111 + HA_realP_11[9]*d311 + HA_realP_11[12]*d511 + HA_realP_11[13]*d111 + HA_realP_11[14]*d311 + HA_realP_11[15]*d711 + HA_realP_11[16]*d311;
    double sum3Re_P1_1 = HA_realP1_1[0]*d33_1+ HA_realP1_1[4]*d13_1+ HA_realP1_1[9]*d33_1+ HA_realP1_1[12]*d53_1+ HA_realP1_1[13]*d13_1+ HA_realP1_1[14]*d33_1+ HA_realP1_1[15]*d73_1+ HA_realP1_1[16]*d33_1;
    double sum3Re_P_1_1= HA_realP_1_1[0]*d331+ HA_realP_1_1[4]*d131+ HA_realP_1_1[9]*d331+ HA_realP_1_1[12]*d531+ HA_realP_1_1[13]*d131+ HA_realP_1_1[14]*d331+ HA_realP_1_1[15]*d731+ HA_realP_1_1[16]*d331;

    double sum3Re_0M11  = HA0_realM11[0]*d311  + HA0_realM11[4]* d111 + HA0_realM11[9]* d311 + HA0_realM11[12]* d511 + HA0_realM11[13]* d111 + HA0_realM11[14]* d311 + HA0_realM11[15]*d711 + HA0_realM11[16]* d311;
    double sum3Re_0M_11 = HA0_realM_11[0]*d31_1+ HA0_realM_11[4]*d11_1+ HA0_realM_11[9]*d31_1+ HA0_realM_11[12]*d51_1+ HA0_realM_11[13]*d11_1+ HA0_realM_11[14]*d31_1+ HA0_realM_11[15]*d71_1+ HA0_realM_11[16]*d31_1;
    double sum3Re_0M1_1 = HA0_realM1_1[0]*d31_1+ HA0_realM1_1[4]*d11_1+ HA0_realM1_1[9]*d31_1+ HA0_realM1_1[12]*d51_1+ HA0_realM1_1[13]*d11_1+ HA0_realM1_1[14]*d31_1+ HA0_realM1_1[15]*d71_1+ HA0_realM1_1[16]*d31_1;
    double sum3Re_0M_1_1= HA0_realM_1_1[0]*d311+ HA0_realM_1_1[4]*d111+ HA0_realM_1_1[9]*d311+ HA0_realM_1_1[12]*d511+ HA0_realM_1_1[13]*d111+ HA0_realM_1_1[14]*d311+ HA0_realM_1_1[15]*d711+ HA0_realM_1_1[16]*d311;

    double sum3Re_0P11  = HA0_realP11[0]*d311  + HA0_realP11[4]* d111 + HA0_realP11[9]* d311 + HA0_realP11[12]* d511 + HA0_realP11[13]* d111 + HA0_realP11[14]* d311 + HA0_realP11[15]*d711 + HA0_realP11[16]* d311;
    double sum3Re_0P_11 = HA0_realP_11[0]*d31_1+ HA0_realP_11[4]*d11_1+ HA0_realP_11[9]*d31_1+ HA0_realP_11[12]*d51_1+ HA0_realP_11[13]*d11_1+ HA0_realP_11[14]*d31_1+ HA0_realP_11[15]*d71_1+HA0_realP_11[16]*d31_1;
    double sum3Re_0P1_1 = HA0_realP1_1[0]*d31_1+ HA0_realP1_1[4]*d11_1+ HA0_realP1_1[9]*d31_1+ HA0_realP1_1[12]*d51_1+ HA0_realP1_1[13]*d11_1+ HA0_realP1_1[14]*d31_1+ HA0_realP1_1[15]*d71_1+ HA0_realP1_1[16]*d31_1;
    double sum3Re_0P_1_1= HA0_realP_1_1[0]*d311+ HA0_realP_1_1[4]*d111+ HA0_realP_1_1[9]*d311+ HA0_realP_1_1[12]*d511+ HA0_realP_1_1[13]*d111+ HA0_realP_1_1[14]*d311+ HA0_realP_1_1[15]*d711+ HA0_realP_1_1[16]*d311;

    double sum3Im_M11  = HA_ImM11[0]*  d331 + HA_ImM11[4]* d131 + HA_ImM11[9]*  d331 + HA_ImM11[12]* d531 + HA_ImM11[13]* d131 + HA_ImM11[14]* d331 + HA_ImM11[15]*d731 + HA_ImM11[16]* d331 ;
    double sum3Im_M_11 = HA_ImM_11[0]* d33_1+ HA_ImM_11[4]*d13_1+ HA_ImM_11[9]*d33_1 + HA_ImM_11[12]*d53_1 +HA_ImM_11[13]*d13_1+ HA_ImM_11[14]*d33_1 + HA_ImM_11[15]*d73_1+ HA_ImM_11[16]*d33_1;
    double sum3Im_M1_1 = HA_ImM1_1[0]* d311 + HA_ImM1_1[4]*d111 + HA_ImM1_1[9]* d311 + HA_ImM1_1[12]*d511 + HA_ImM1_1[13]*d111 + HA_ImM1_1[14]* d311 + HA_ImM1_1[15]*d711+ HA_ImM1_1[16]* d311;
    double sum3Im_M_1_1= HA_ImM_1_1[0]*d31_1+ HA_ImM_1_1[4]*d11_1+HA_ImM_1_1[9]*d31_1+ HA_ImM_1_1[12]*d51_1+HA_ImM_1_1[13]*d11_1+ HA_ImM_1_1[14]*d31_1+HA_ImM_1_1[15]*d71_1+ HA_ImM_1_1[16]*d31_1;

    double sum3Im_P11  = HA_ImP11[0]*d31_1 + HA_ImP11[4]*d11_1 + HA_ImP11[9]*d31_1 + HA_ImP11[12]*d51_1 + HA_ImP11[13]*d11_1 + HA_ImP11[14]*d31_1 + HA_ImP11[15]*d71_1 + HA_ImP11[16]*d31_1;
    double sum3Im_P_11 = HA_ImP_11[0]*d311 + HA_ImP_11[4]*d111 + HA_ImP_11[9]*d311 + HA_ImP_11[12]*d511 + HA_ImP_11[13]*d111 + HA_ImP_11[14]*d311 + HA_ImP_11[15]*d711+ HA_ImP_11[16]*d311;
    double sum3Im_P1_1 = HA_ImP1_1[0]*d33_1+ HA_ImP1_1[4]*d13_1+ HA_ImP1_1[9]*d33_1+ HA_ImP1_1[12]*d53_1+ HA_ImP1_1[13]*d13_1+ HA_ImP1_1[14]*d33_1+ HA_ImP1_1[15]*d73_1+ HA_ImP1_1[16]*d33_1;
    double sum3Im_P_1_1= HA_ImP_1_1[0]*d331+ HA_ImP_1_1[4]*d131+ HA_ImP_1_1[9]*d331+ HA_ImP_1_1[12]*d531+ HA_ImP_1_1[13]*d131+ HA_ImP_1_1[14]*d331+ HA_ImP_1_1[15]*d731+ HA_ImP_1_1[16]*d331;

    double sum3Im_0M11  = HA0_ImM11[0]*d311  + HA0_ImM11[4]* d111 + HA0_ImM11[9]* d311 + HA0_ImM11[12]* d511 + HA0_ImM11[13]* d111 + HA0_ImM11[14]* d311 + HA0_ImM11[15]*d711 + HA0_ImM11[16]* d311;
    double sum3Im_0M_11 = HA0_ImM_11[0]*d31_1+ HA0_ImM_11[4]*d11_1+ HA0_ImM_11[9]*d31_1+ HA0_ImM_11[12]*d51_1+ HA0_ImM_11[13]*d11_1+ HA0_ImM_11[14]*d31_1+ HA0_ImM_11[15]*d71_1+ HA0_ImM_11[16]*d31_1;
    double sum3Im_0M1_1 = HA0_ImM1_1[0]*d31_1+ HA0_ImM1_1[4]*d11_1+ HA0_ImM1_1[9]*d31_1+ HA0_ImM1_1[12]*d51_1+ HA0_ImM1_1[13]*d11_1+ HA0_ImM1_1[14]*d31_1+ HA0_ImM1_1[15]*d71_1+ HA0_ImM1_1[16]*d31_1;
    double sum3Im_0M_1_1= HA0_ImM_1_1[0]*d311+ HA0_ImM_1_1[4]*d111+ HA0_ImM_1_1[9]*d311+ HA0_ImM_1_1[12]*d511+ HA0_ImM_1_1[13]*d111+ HA0_ImM_1_1[14]*d311+ HA0_ImM_1_1[15]*d711+ HA0_ImM_1_1[16]*d311;

    double sum3Im_0P11  = HA0_ImP11[0]*d311  + HA0_ImP11[4]* d111 + HA0_ImP11[9]* d311 + HA0_ImP11[12]* d511 + HA0_ImP11[13]* d111 + HA0_ImP11[14]* d311 + HA0_ImP11[15]*d711 + HA0_ImP11[16]* d311;
    double sum3Im_0P_11 = HA0_ImP_11[0]*d31_1+ HA0_ImP_11[4]*d11_1+ HA0_ImP_11[9]*d31_1+ HA0_ImP_11[12]*d51_1+ HA0_ImP_11[13]*d11_1+ HA0_ImP_11[14]*d31_1+ HA0_ImP_11[15]*d71_1+ HA0_ImP_11[16]*d31_1;
    double sum3Im_0P1_1 = HA0_ImP1_1[0]*d31_1+ HA0_ImP1_1[4]*d11_1+ HA0_ImP1_1[9]*d31_1+ HA0_ImP1_1[12]*d51_1+ HA0_ImP1_1[13]*d11_1+ HA0_ImP1_1[14]*d31_1+ HA0_ImP1_1[15]*d71_1+ HA0_ImP1_1[16]*d31_1;
    double sum3Im_0P_1_1= HA0_ImP_1_1[0]*d311+ HA0_ImP_1_1[4]*d111+ HA0_ImP_1_1[9]*d311+ HA0_ImP_1_1[12]*d511+ HA0_ImP_1_1[13]*d111+ HA0_ImP_1_1[14]*d311+ HA0_ImP_1_1[15]*d711+ HA0_ImP_1_1[16]*d311;


    ///////////////////////////////////////////////////////////////////////////////////////////
    double xsec_cpp = 

      ((0.974)*(0.974)*(1./Gvcm)*(1./Gvcm)*(1e13)*G_F*G_F/(64.*2.*pi*pi*pi*pi)) * (abs_mom_q/(abs_mom_k_L*abs_mom_k_L))
      *(  
          C_T_plus_square*( 
            3.*(sum3Im_M11*sum3Im_M11 + sum3Im_M_11*sum3Im_M_11 + sum3Im_M1_1*sum3Im_M1_1 + sum3Im_M_1_1*sum3Im_M_1_1) 

            +  pow( ((F_minus11   - G_minus11  ) + sqrt(3.)*sum3Re_M11) , 2)
            +  pow( ((F_minus_11  - G_minus_11 ) + sqrt(3.)*sum3Re_M_11) , 2) 
            +  pow( ((F_minus1_1  - G_minus1_1 ) + sqrt(3.)*sum3Re_M1_1 ) , 2)
            +  pow( ((F_minus_1_1 - G_minus_1_1) + sqrt(3.)*sum3Re_M_1_1 ) , 2)
            )
          + C_T_minus_square*(
            3.*(sum3Im_P11*sum3Im_P11 + sum3Im_P_11*sum3Im_P_11 + sum3Im_P1_1*sum3Im_P1_1 + sum3Im_P_1_1*sum3Im_P_1_1) 

            +  pow( ((F_plus11   - G_plus11  ) + sqrt(3.)*sum3Re_P11) , 2)
            +  pow( ((F_plus_11  - G_plus_11 ) + sqrt(3.)*sum3Re_P_11) , 2) 
            +  pow( ((F_plus1_1  - G_plus1_1 ) + sqrt(3.)*sum3Re_P1_1 ) , 2)
            +  pow( ((F_plus_1_1 - G_plus_1_1) + sqrt(3.)*sum3Re_P_1_1 ) , 2)
            )
          + C_S_minus_square*(
            3.*(sum3Im_0M11*sum3Im_0M11 + sum3Im_0M_11*sum3Im_0M_11 + sum3Im_0M1_1*sum3Im_0M1_1 + sum3Im_0M_1_1*sum3Im_0M_1_1)

            +  pow( ((F_zero_minus11   - G_zero_minus11  ) + sqrt(3.)*sum3Re_0M11) , 2)
            +  pow( ((F_zero_minus_11  - G_zero_minus_11 ) + sqrt(3.)*sum3Re_0M_11) , 2) 
            +  pow( ((F_zero_minus1_1  - G_zero_minus1_1 ) + sqrt(3.)*sum3Re_0M1_1 ) , 2)
            +  pow( ((F_zero_minus_1_1 - G_zero_minus_1_1) + sqrt(3.)*sum3Re_0M_1_1 ) , 2)
            )
          + C_S_plus_square*(
              3.*(sum3Im_0P11*sum3Im_0P11 + sum3Im_0P_11*sum3Im_0P_11 + sum3Im_0P1_1*sum3Im_0P1_1 + sum3Im_0P_1_1*sum3Im_0P_1_1)

              +  pow( ((F_zero_plus11   - G_zero_plus11  ) + sqrt(3.)*sum3Re_0P11) , 2)
              +  pow( ((F_zero_plus_11  - G_zero_plus_11 ) + sqrt(3.)*sum3Re_0P_11) , 2) 
              +  pow( ((F_zero_plus1_1  - G_zero_plus1_1 ) + sqrt(3.)*sum3Re_0P1_1 ) , 2)
              +  pow( ((F_zero_plus_1_1 - G_zero_plus_1_1) + sqrt(3.)*sum3Re_0P_1_1 ) , 2)
              )

          + 2.*cos (phi)*(  

              + sqrt(C_T_minus_square)*sqrt(C_S_minus_square)  *( 

                3.*(sum3Im_M11*sum3Im_0M11 + sum3Im_M_11*sum3Im_0M_11 + sum3Im_M1_1*sum3Im_0M1_1 + sum3Im_M_1_1*sum3Im_0M_1_1) 

                +   ((F_minus11   - G_minus11  ) + sqrt(3.)*sum3Re_M11)* ((F_zero_minus11   - G_zero_minus11  ) + sqrt(3.)*sum3Re_0M11)
                +   ((F_minus_11  - G_minus_11 ) + sqrt(3.)*sum3Re_M_11)* ((F_zero_minus_11  - G_zero_minus_11 ) + sqrt(3.)*sum3Re_0M_11)
                +   ((F_minus1_1  - G_minus1_1 ) + sqrt(3.)*sum3Re_M1_1 )*((F_zero_minus1_1  - G_zero_minus1_1 ) + sqrt(3.)*sum3Re_0M1_1 ) 
                +   ((F_minus_1_1 - G_minus_1_1) + sqrt(3.)*sum3Re_M_1_1)* ((F_zero_minus_1_1 - G_zero_minus_1_1) + sqrt(3.)*sum3Re_0M_1_1 )
                )
              + sqrt(C_T_plus_square)*sqrt(C_S_minus_square)*( 

                3.*(sum3Im_P11*sum3Im_0M11 + sum3Im_P_11*sum3Im_0M_11 + sum3Im_P1_1*sum3Im_0M1_1 + sum3Im_P_1_1*sum3Im_0M_1_1) 

                +   ((F_plus11   - G_plus11  ) + sqrt(3.)*sum3Re_P11)*((F_zero_minus11   - G_zero_minus11  ) + sqrt(3.)*sum3Re_0M11)
                +   ((F_plus_11  - G_plus_11 ) + sqrt(3.)*sum3Re_P_11)*((F_zero_minus_11  - G_zero_minus_11 ) + sqrt(3.)*sum3Re_0M_11)
                +   ((F_plus1_1  - G_plus1_1 ) + sqrt(3.)*sum3Re_P1_1)*((F_zero_minus1_1  - G_zero_minus1_1 ) + sqrt(3.)*sum3Re_0M1_1 )
                +   ((F_plus_1_1 - G_plus_1_1) + sqrt(3.)*sum3Re_P_1_1)*((F_zero_minus_1_1 - G_zero_minus_1_1) + sqrt(3.)*sum3Re_0M_1_1 ) 
                )
              + sqrt(C_T_minus_square)*sqrt(C_S_plus_square)  *( 

                3.*(sum3Im_M11*sum3Im_0P11 + sum3Im_M_11*sum3Im_0P_11 + sum3Im_M1_1*sum3Im_0P1_1 + sum3Im_M_1_1*sum3Im_0P_1_1) 

                +   ((F_minus11   - G_minus11  ) + sqrt(3.)*sum3Re_M11)* ((F_zero_plus11   - G_zero_plus11  ) + sqrt(3.)*sum3Re_0P11)
                +   ((F_minus_11  - G_minus_11 ) + sqrt(3.)*sum3Re_M_11)* ((F_zero_plus_11  - G_zero_plus_11 ) + sqrt(3.)*sum3Re_0P_11)
                +   ((F_minus1_1  - G_minus1_1 ) + sqrt(3.)*sum3Re_M1_1 )*((F_zero_plus1_1  - G_zero_plus1_1 ) + sqrt(3.)*sum3Re_0P1_1 ) 
                +   ((F_minus_1_1 - G_minus_1_1) + sqrt(3.)*sum3Re_M_1_1)* ((F_zero_plus_1_1 - G_zero_plus_1_1) + sqrt(3.)*sum3Re_0P_1_1 )
                )
              + sqrt(C_T_plus_square)*sqrt(C_S_plus_square)*( 

                  3.*(sum3Im_P11*sum3Im_0P11 + sum3Im_P_11*sum3Im_0P_11 + sum3Im_P1_1*sum3Im_0P1_1 + sum3Im_P_1_1*sum3Im_0P_1_1) 

                  +   ((F_plus11   - G_plus11  ) + sqrt(3.)*sum3Re_P11)*((F_zero_plus11   - G_zero_plus11  ) + sqrt(3.)*sum3Re_0P11)
                  +   ((F_plus_11  - G_plus_11 ) + sqrt(3.)*sum3Re_P_11)*((F_zero_plus_11  - G_zero_plus_11 ) + sqrt(3.)*sum3Re_0P_11)
                  +   ((F_plus1_1  - G_plus1_1 ) + sqrt(3.)*sum3Re_P1_1)*((F_zero_plus1_1  - G_zero_plus1_1 ) + sqrt(3.)*sum3Re_0P1_1 )
                  +   ((F_plus_1_1 - G_plus_1_1) + sqrt(3.)*sum3Re_P_1_1)*((F_zero_plus_1_1 - G_zero_plus_1_1) + sqrt(3.)*sum3Re_0P_1_1 ) 
                  )

              )

              + 2.*sin (phi) *(       
                  -sqrt(3.)* sqrt(C_T_minus_square)*sqrt(C_S_minus_square)  *( 

                    sum3Im_M11*((F_zero_minus11   - G_zero_minus11  ) + sqrt(3.)*sum3Re_0M11)

                    + sum3Im_M_11*((F_zero_minus_11  - G_zero_minus_11 ) + sqrt(3.)*sum3Re_0M_11)

                    + sum3Im_M1_1*((F_zero_minus1_1  - G_zero_minus1_1 ) + sqrt(3.)*sum3Re_0M1_1 )

                    + sum3Im_M_1_1*((F_zero_minus_1_1 - G_zero_minus_1_1) + sqrt(3.)*sum3Re_0M_1_1 )


                    -   ((F_minus11   - G_minus11  ) + sqrt(3.)*sum3Re_M11)* sum3Im_0M11
                    -   ((F_minus_11  - G_minus_11 ) + sqrt(3.)*sum3Re_M_11)*  sum3Im_0M_11
                    -   ((F_minus1_1  - G_minus1_1 ) + sqrt(3.)*sum3Re_M1_1 )*sum3Im_0M1_1  
                    -   ((F_minus_1_1 - G_minus_1_1) + sqrt(3.)*sum3Re_M_1_1)* sum3Im_0M_1_1
                    )

                  + sqrt(3.)*sqrt(C_T_plus_square)*sqrt(C_S_minus_square)*( 

                    sum3Im_P11*((F_zero_minus11   - G_zero_minus11  ) + sqrt(3.)*sum3Re_0M11)                 
                    + sum3Im_P_11*((F_zero_minus_11  - G_zero_minus_11 ) + sqrt(3.)*sum3Re_0M_11)                 
                    + sum3Im_P1_1*((F_zero_minus1_1  - G_zero_minus1_1 ) + sqrt(3.)*sum3Re_0M1_1 )                 
                    + sum3Im_P_1_1*((F_zero_minus_1_1 - G_zero_minus_1_1) + sqrt(3.)*sum3Re_0M_1_1 )

                    -   ((F_plus11   - G_plus11  ) + sqrt(3.)*sum3Re_P11)*sum3Im_0M11 
                    -   ((F_plus_11  - G_plus_11 ) + sqrt(3.)*sum3Re_P_11)*sum3Im_0M_11
                    -   ((F_plus1_1  - G_plus1_1 ) + sqrt(3.)*sum3Re_P1_1)*sum3Im_0M1_1 
                    -   ((F_plus_1_1 - G_plus_1_1) + sqrt(3.)*sum3Re_P_1_1)*sum3Im_0M_1_1 
                    )

                  - sqrt(3.)*sqrt(C_T_minus_square)*sqrt(C_S_plus_square)  *( 

                      sum3Im_M11*((F_zero_plus11   - G_zero_plus11  ) + sqrt(3.)*sum3Re_0P11)                  
                      + sum3Im_M_11*((F_zero_plus_11  - G_zero_plus_11 ) + sqrt(3.)*sum3Re_0P_11)                 
                      + sum3Im_M1_1*((F_zero_plus1_1  - G_zero_plus1_1 ) + sqrt(3.)*sum3Re_0P1_1 )                   
                      + sum3Im_M_1_1*((F_zero_plus_1_1 - G_zero_plus_1_1) + sqrt(3.)*sum3Re_0P_1_1 )                   

                      -   ((F_minus11   - G_minus11  ) + sqrt(3.)*sum3Re_M11)* sum3Im_0P11 
                      -   ((F_minus_11  - G_minus_11 ) + sqrt(3.)*sum3Re_M_11)*  sum3Im_0P_11 
                      -   ((F_minus1_1  - G_minus1_1 ) + sqrt(3.)*sum3Re_M1_1 )*sum3Im_0P1_1 
                      -   ((F_minus_1_1 - G_minus_1_1) + sqrt(3.)*sum3Re_M_1_1)* sum3Im_0P_1_1
                      )

                  + sqrt(3.)*sqrt(C_T_plus_square)*sqrt(C_S_plus_square)*( 

                      sum3Im_P11*((F_zero_plus11   - G_zero_plus11  ) + sqrt(3.)*sum3Re_0P11)                  
                      + sum3Im_P_11*((F_zero_plus_11  - G_zero_plus_11 ) + sqrt(3.)*sum3Re_0P_11)                 
                      + sum3Im_P1_1*((F_zero_plus1_1  - G_zero_plus1_1 ) + sqrt(3.)*sum3Re_0P1_1 )                  
                      + sum3Im_P_1_1*((F_zero_plus_1_1 - G_zero_plus_1_1) + sqrt(3.)*sum3Re_0P_1_1 )                 

                      -   ((F_plus11   - G_plus11  ) + sqrt(3.)*sum3Re_P11)*sum3Im_0P11
                      -   ((F_plus_11  - G_plus_11 ) + sqrt(3.)*sum3Re_P_11)*sum3Im_0P_11 
                      -   ((F_plus1_1  - G_plus1_1 ) + sqrt(3.)*sum3Re_P1_1)*sum3Im_0P1_1
                      -   ((F_plus_1_1 - G_plus_1_1) + sqrt(3.)*sum3Re_P_1_1)*sum3Im_0P_1_1 
                      )
                  )

                  + 2.*cos(2.*phi)*sqrt(C_T_minus_square)*sqrt(C_T_plus_square)  *( 

                      3.*(sum3Im_M11*sum3Im_P11 + sum3Im_M_11*sum3Im_P_11 + sum3Im_M1_1*sum3Im_P1_1 + sum3Im_M_1_1*sum3Im_P_1_1) 

                      +   ((F_minus11   - G_minus11  ) + sqrt(3.)*sum3Re_M11)* ((F_plus11   - G_plus11  ) + sqrt(3.)*sum3Re_P11)
                      +   ((F_minus_11  - G_minus_11 ) + sqrt(3.)*sum3Re_M_11)* ((F_plus_11  - G_plus_11 ) + sqrt(3.)*sum3Re_P_11)
                      +   ((F_minus1_1  - G_minus1_1 ) + sqrt(3.)*sum3Re_M1_1 )*((F_plus1_1  - G_plus1_1 ) + sqrt(3.)*sum3Re_P1_1 ) 
                      +   ((F_minus_1_1 - G_minus_1_1) + sqrt(3.)*sum3Re_M_1_1)* ((F_plus_1_1 - G_plus_1_1) + sqrt(3.)*sum3Re_P_1_1 )
                      )

                  - 2.* sin (2.*phi)*sqrt(3.)* sqrt(C_T_minus_square)*sqrt(C_T_plus_square)  *( 

                      sum3Im_M11*((F_plus11   - G_plus11  ) + sqrt(3.)*sum3Re_P11)                   
                      + sum3Im_M_11*((F_plus_11  - G_plus_11 ) + sqrt(3.)*sum3Re_P_11)                  
                      + sum3Im_M1_1*((F_plus1_1  - G_plus1_1 ) + sqrt(3.)*sum3Re_P1_1 )                  
                      + sum3Im_M_1_1*((F_plus_1_1 - G_plus_1_1) + sqrt(3.)*sum3Re_P_1_1 )


                      -   ((F_minus11   - G_minus11  ) + sqrt(3.)*sum3Re_M11)* sum3Im_P11
                      -   ((F_minus_11  - G_minus_11 ) + sqrt(3.)*sum3Re_M_11)*  sum3Im_P_11
                      -   ((F_minus1_1  - G_minus1_1 ) + sqrt(3.)*sum3Re_M1_1 )*sum3Im_P1_1  
                      -   ((F_minus_1_1 - G_minus_1_1) + sqrt(3.)*sum3Re_M_1_1)* sum3Im_P_1_1
                      )
                  );
    *diff_Xsec = float(xsec_cpp);

  }





