#ifndef __MKCONS_IS_DEFINED__
#define __MKCONS_IS_DEFINED__

// An anonymous namespace for Minoo's shared constants for the mk_imodeXY.cc
// Implementation intentionally made to mimic Rein-Sehgal code in NEUT
// The parameters are now read from the NEUT card file instead of this namespace (e.g. MARES, CA5(0))

namespace {

  // Make these the same as in rscons.h to avoid running into numerical issues
  // Mass of nucleon (GeV)
  const double M = 0.938919;
  // Mass of pion (GeV)
  const double m_pi = 0.138;

  // ******************
  // Variables of model which are tunable
  // Mass of resonance background
  //const double MAR = 0.628524;
  // CA5(0) axial form factor: now set with NEUT card file for each imode
  //const double C50 = 1.02683;
  // Axial mass: now set with NEUT card file for each imode
  //const double M_A = 1.05;
  // Vector mass: now set with NEUT card file for each imode
  //const double M_V = 0.84;
  // ******************

  // Mass of rho
  const double m_rho= 0.7758;

  // Weinberg angle
  const double sin2Wein = 0.231;

  const double F0_A= 1.26;
  const double F0_1=1.0;
  const double F0_pi= 1.0;
  const double F0_2=1.85;
  const double F0_rho= 1.;
  const double f_pi=0.093;
  const double g_A= 1.26;
  const double g_NN=13.50;//g_NN\pi
  const double G_F= 1.1664e-5 ;//GeV-2
  const double Omega= 1.05 ;//GeV+2
  const double Z= 3./4. ;

  // Just pi, could use cmath here intead
  const double pi = 3.141592653589793;
  const double Gvcm= 5.067731239;

  // Specifics of the resonances
  // The resonance masses
  const double MR[17] = {1.232, 1.430, 1.515, 1.535, 1.63, 1.655, 1.675, 1.685, 1.7, 1.7, 1.71, 1.72, 1.88, 1.89, 1.92, 1.93, 1.6};
  // The resonance widths
  const double RWA[17] = {0.117, 0.35, 0.115, 0.15, 0.14, 0.14, 0.15, 0.13, 0.15, 0.3, 0.1, 0.25, 0.33, 0.28, 0.26, 0.285, 0.32};
  // The resonance branching ratios to 1pi
  const double BRA[17] = {1., 0.65, 0.6, 0.45, 0.25, 0.7, 0.4, 0.67, 0.12, 0.15, 0.12, 0.11, 0.12, 0.22, 0.12, 0.4, 0.18};
  // The resonance normalizations
  // Minoo upd Sep 2017
  const double NA[17] = {1.01763, 1.13739, 0.990136, 0.961495, 0.968959,0.970112 , 0.98815, 1.04593, 0.98157, 1.15457, 0.859043,0.92719 , 1.24422, 0.915705, 0.903209, 1.17743, 1.01075};  
  //const double NA[17] = {1.01762, 0.841642, 0.990136, 0.936202, 0.947601,0.949313 , 0.98815, 1.04593, 0.98157, 1.15457, 0.858881,0.927106 , 1.24422, 0.915585, 0.903074, 1.17743, 1.};  

  // RS signs for each resonance
  //const int Dsgn[17] ={1, 1, -1, -1, 1, 1, 1, 1, -1, 1, -1, 1, -1, -1, 1, 1};
  // MK signs for resonances
  // Minoo upd Sep 2017
  const int Dsgn[17] ={1, -1, -1, -1, -1, 1, -1, 1, -1, 1, -1, -1, 1, -1, -1, 1, 1};

  const int Cjsgn_plus[17]  ={1,-1,-1, 1, 1, 1, 1,-1, -1,-1,-1, 1, -1, -1, 1, 1,1};
  const int Cjsgn_minus[17] ={1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,1};

  const int Jsgn[17] ={1,-1,-1, 1, 1, 1, 1,-1, -1,-1,-1, 1, -1, -1, 1, 1,1};

  const int LP[17] = {3, 3, 5, 1, 1, 1, 5, 7, 5, 5, 3, 3, 7, 3, 3, 7, 3}; //2l+1
  const int JP[17] = {2, 1, 2, 1, 1, 1, 3, 3, 2, 2, 1, 2, 3, 1, 2, 4, 2};//2j+1/2

  const int Del[17] ={1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 ,1 ,1 ,1, 1,1};

}

#endif
