void
fill_histos_numu()
{
  Double_t normalization;

  Int_t i, j;
  Int_t nevents;

  gSystem->Load("../../../neutclass/neutvtx.so");
  gSystem->Load("../../../neutclass/neutpart.so");
  gSystem->Load("../../../neutclass/neutfsipart.so");
  gSystem->Load("../../../neutclass/neutfsivert.so");
  gSystem->Load("../../../neutclass/neutnucfsistep.so");
  gSystem->Load("../../../neutclass/neutnucfsivert.so");
  gSystem->Load("../../../neutclass/neutvect.so");
  
  TTree  *tn;
  NeutVtx *nvtx;
  NeutVect *nvect;

  char flux_fname[32]     ="numu_fhc_o.root\0";
  char rate_hist_name[32] ="evtrt_numu\0";
  char flux_hist_name[32] ="flux_numu\0";

  char data_fnames[64]    ="neut_minerva_numu_K*.000.root\0";
  char out_fname[64]      ="h_neut_minerva_numu_K_RPA.root\0";


  TFile f_flux(flux_fname);
  if (!f_flux){
	cout << "Failed to open flux output file " << flux_fname << endl;
	return;
  }
  
  TH1D *h_evtrt = (TH1D *)(f_flux.Get(rate_hist_name));
  if (!h_evtrt){
	cout << "Failed to find eventrate histogram " << rate_hist_name << endl;
	return;
  }

  TH1D *h_flux = (TH1D *)(f_flux.Get(flux_hist_name));
  if (!h_flux){
	cout << "Failed to find flux histogram " << flux_hist_name << endl;
	return;
  }

  normalization  = h_evtrt->GetSumOfWeights() / h_flux->GetSumOfWeights();

  TChain ch("neuttree");
  ch.Add("neut_minerva_numu_K_RPA.???.root");

  //  tn = (TTree *)(ch.Get("neuttree"));

  nvtx = new NeutVtx();
  ch.SetBranchAddress("vertexbranch",&nvtx);

  nvect = new NeutVect();
  ch.SetBranchAddress("vectorbranch",&nvect);

  nevents = ch.GetEntries();
  cout << "# of events = " << nevents << endl;

  normalization = normalization / (double)nevents;

  ////////////////////////////////////////////////////////////////
  TFile *f2;
  f2 = new TFile(out_fname,"RECREATE");

  f2->cd();

  ////////////////////////////////////////////////////////////////
  Double_t binsize = 0.1; /* 0.1 GeV */
  Double_t weight;

  weight = 1. / binsize * normalization;

  TH1D *h_EK_all;
  TH1D *h_EK_all_NCK;
  h_EK_all    = new TH1D("EK_all",
						 "Kinetic energy of K+(all)",
						 50,0.,5);
  h_EK_all_NCK= new TH1D("EK_all_NCK",
						 "Kinetic energy of K+(all, only NC K+ prod.)",
						 50,0.,5);

  TH1D *h_EK_lowEav;
  TH1D *h_EK_lowEav_NCK;
  h_EK_lowEav    = 
	new TH1D("EK_lowEav",
			 "Kinetic energy of K+(E_Availe<1GeV)",
			 50,0.,0.5);  
  h_EK_lowEav_NCK= 
	new TH1D("EK_lowEav_NCK",
			 "Kinetic energy of K+(E_Availe<1GeV, only NC K+ prod.)",
			 50,0.,0.5);  

  ////////////////////////////////////////////////////////////////
  Int_t ip;
  Int_t mode;
  Int_t idx;

  Double_t Eavail;
  Double_t Ekine;
  Double_t xmass;

  Double_t Enu,Emu,Pmu;
  Double_t cosmu;
  Double_t xmmu;  
  Double_t Q2,q0,q3;

  Int_t    k_idx;
  Int_t    E_k;

  for ( j = 0 ; j < nevents ; j++ ){
	ch.GetEntry(j);

	Eavail = 0;

	mode = nvect->Mode;

	/* skip CC interactions */
	if ( abs(mode) < 30 ){
	  continue;
	}

	if ( abs(mode) != 2 ){
	  idx = 2;
	}else{
	  idx = 3;
	}

	/* search for most energetic K+ : 321 */
	k_idx = 0;
	E_k   = 0.;
	for ( i = 2 ; i < nvect->Npart() ; i++ ){
	  if ( ((nvect->PartInfo(i))->fIsAlive) ){
		ip = (nvect->PartInfo(i))->fPID;
		if ( ip == 321 ){
		  if (( nvect->PartInfo(i))->fP.E() > E_k ){
			E_k = (nvect->PartInfo(i))->fP.E();
			k_idx = i;
		  }
		}
	  }
	}		

	if ( k_idx == 0 ) continue;

	for ( i = idx+1 ; i < nvect->Npart() ; i++ ){
	  if ( ((nvect->PartInfo(i))->fIsAlive) ){
		ip = (nvect->PartInfo(i))->fPID;
		if ( (ip      == 2212)|| /* proton */
			 (abs(ip) ==  211)|| /* pi+- */
			 (ip      ==  310))  /* K0 short : assuming decay into pi+- */ {
		  Ekine = nvect->PartInfo(i)->fP.E()-nvect->PartInfo(i)->fMass;
		  Eavail += (Ekine/1000.);
		} else if (( abs(ip) == 111 )|| /* pi0 */ 
				   ( abs(ip) == 22  )|| /* gamma */
				   ( abs(ip) == 12  )){ /* electron */
		  Eavail += (nvect->PartInfo(i)->fP.E())/1000.0;
		} else if ( abs(ip) == 3122 )   /* lambda */     {
		  Ekine = nvect->PartInfo(i)->fP.E()-nvect->PartInfo(i)->fMass;
		  Ekine = Ekine + 200; /* take into account the decay */
		  Eavail += (Ekine/1000.);
		}
	  }
	}
	

	Enu  = (nvect->PartInfo(0))->fP.E()/1000.;
	Emu  = ((nvect->PartInfo(idx))->fP.E()/1000.);
	Pmu  = ((nvect->PartInfo(idx))->fP.P()/1000.);
	xmmu = ((nvect->PartInfo(idx))->fMass/1000.);
	
	cosmu = (
			 (nvect->PartInfo(0))->fP.Px()*(nvect->PartInfo(idx))->fP.Px() +
			 (nvect->PartInfo(0))->fP.Py()*(nvect->PartInfo(idx))->fP.Py() +
			 (nvect->PartInfo(0))->fP.Pz()*(nvect->PartInfo(idx))->fP.Pz()
			 )/
	  ((nvect->PartInfo(0))->fP.P() * (nvect->PartInfo(idx))->fP.P());
	
	q0 =(nvect->PartInfo(0))->fP.E() - (nvect->PartInfo(idx))->fP.E();
	q0 = q0 / 1000.;
	
	Q2 = 2 * Enu * ( Emu - Pmu * cosmu ) - xmmu*xmmu;
	
	q3 = sqrt(Q2 + q0*q0);
	
	if ( abs((nvect->PartInfo(k_idx))->fPID) != 321 ){
	  cout << "ip is incorret??" << endl;
	  cout << (nvect->PartInfo(k_idx))->fPID << endl;
	}

	Ekine = nvect->PartInfo(k_idx)->fP.E()-nvect->PartInfo(k_idx)->fMass;
	Ekine = Ekine /1000.;

	//	cout << weight << endl;

	h_EK_all->Fill(Ekine,weight);
	if ( abs(mode) == 45 ){
	  h_EK_all_NCK->Fill(Ekine,weight);

	  cout << "Event #        :" << nvect->EventNo << "\n";
	  cout << "Target A       :" << nvect->TargetA << "\n";
	  cout << "Target Z       :" << nvect->TargetA << "\n";
	  cout << "VNuclIni       :" << nvect->VNuclIni << "\n";
	  cout << "VNuclFin       :" << nvect->VNuclFin << "\n";
	  cout << "PF Surface     :" << nvect->PFSurf   << "\n";
	  cout << "PF Maximum     :" << nvect->PFMax    << "\n";
	  cout << "Flux ID        :" << nvect->FluxID   << "\n";
	  
	  cout << "Intr. mode     :" << nvect->Mode   << "\n";
	  
	  for ( i = 0 ; i < nvect->Npart() ; i++ ){
		cout << "i=" << i << "\n";
		cout << "Vertex         =" << nvect->VertexID(i) << "\n";
		cout << "Parent Index   =" << nvect->ParentIdx(i) << "\n";
		
		cout << "Particle Code  = " << (nvect->PartInfo(i))->fPID   << "\n";
		cout << "Particle Mass  = " << (nvect->PartInfo(i))->fMass   << "\n";
		cout << "Particle Mom.  =(" << (nvect->PartInfo(i))->fP.Px() << ","
			 << (nvect->PartInfo(i))->fP.Py() << "," 
			 << (nvect->PartInfo(i))->fP.Pz() << ","
			 << (nvect->PartInfo(i))->fP.E()  << ")"
			 << "\n";
		cout << "Particle Flag  = " << (nvect->PartInfo(i))->fIsAlive << "\n";
		cout << "Particle Stat. = " << (nvect->PartInfo(i))->fStatus  << "\n";
		cout << "Particle Pos(1)=(" << (nvect->PartInfo(i))->fPosIni.X() << ","
			 << (nvect->PartInfo(i))->fPosIni.Y() << "," 
			 << (nvect->PartInfo(i))->fPosIni.Z() << ","
			 << (nvect->PartInfo(i))->fPosIni.T()  << ")"
			 << "\n";
		cout << "Particle Pos(2)=(" << (nvect->PartInfo(i))->fPosFin.Px() << ","
			 << (nvect->PartInfo(i))->fPosFin.Y() << "," 
			 << (nvect->PartInfo(i))->fPosFin.Z() << ","
			 << (nvect->PartInfo(i))->fPosFin.T()  << ")"
			 << "\n";
	  }
	}

	if ( Eavail < 1. ){
	  h_EK_lowEav->Fill(Ekine,weight);
	  if ( abs(mode) == 45 ){
		h_EK_lowEav_NCK->Fill(Ekine,weight);
	  }
	}
  }


  /***************************************************************/
  /*
	cout << "Event #        :" << nvect->EventNo << "\n";
	cout << "Target A       :" << nvect->TargetA << "\n";
	cout << "Target Z       :" << nvect->TargetA << "\n";
	cout << "VNuclIni       :" << nvect->VNuclIni << "\n";
	cout << "VNuclFin       :" << nvect->VNuclFin << "\n";
	cout << "PF Surface     :" << nvect->PFSurf   << "\n";
	cout << "PF Maximum     :" << nvect->PFMax    << "\n";
	cout << "Flux ID        :" << nvect->FluxID   << "\n";

	cout << "Intr. mode     :" << nvect->Mode   << "\n";

	for ( i = 0 ; i < nvect->Npart() ; i++ ){
	  cout << "i=" << i << "\n";
	  cout << "Vertex         =" << nvect->VertexID(i) << "\n";
	  cout << "Parent Index   =" << nvect->ParentIdx(i) << "\n";

	  cout << "Particle Code  = " << (nvect->PartInfo(i))->fPID   << "\n";
	  cout << "Particle Mass  = " << (nvect->PartInfo(i))->fMass   << "\n";
	  cout << "Particle Mom.  =(" << (nvect->PartInfo(i))->fP.Px() << ","
	  	   << (nvect->PartInfo(i))->fP.Py() << "," 
		   << (nvect->PartInfo(i))->fP.Pz() << ","
		   << (nvect->PartInfo(i))->fP.E()  << ")"
		   << "\n";
	  cout << "Particle Flag  = " << (nvect->PartInfo(i))->fIsAlive << "\n";
	  cout << "Particle Stat. = " << (nvect->PartInfo(i))->fStatus  << "\n";
	  cout << "Particle Pos(1)=(" << (nvect->PartInfo(i))->fPosIni.X() << ","
	  	   << (nvect->PartInfo(i))->fPosIni.Y() << "," 
		   << (nvect->PartInfo(i))->fPosIni.Z() << ","
		   << (nvect->PartInfo(i))->fPosIni.T()  << ")"
		   << "\n";
	  cout << "Particle Pos(2)=(" << (nvect->PartInfo(i))->fPosFin.Px() << ","
	  	   << (nvect->PartInfo(i))->fPosFin.Y() << "," 
		   << (nvect->PartInfo(i))->fPosFin.Z() << ","
		   << (nvect->PartInfo(i))->fPosFin.T()  << ")"
		   << "\n";
	}
	std::cout << "Event #" << nvtx->EventNo << "\n";

	for (i = 0 ; i < nvtx->Nvtx() ; i++){
	  cout << "i=" << i << "\n";
	  
	  cout << "Vertex Pos(1)=(" << (nvtx->Pos(i))->X() << ","
		   << (nvtx->Pos(i))->Y() << "," 
		   << (nvtx->Pos(i))->Z() << ","
		   << (nvtx->Pos(i))->T()  << ")"
		   << "\n";
	}
  */
  f2->Write();
  exit();
}
  
