void
fill_histos()
{
  /* without RPA */
  const Double_t normalization = 1.9887/7.363*1.e5;
  /* with RPA */
  //const Double_t normalization = 1.9887/7.419*1.e5;

  

  Int_t i, j;
  Int_t nevents;

  gSystem->Load("../../neutclass/neutvtx.so");
  gSystem->Load("../../neutclass/neutpart.so");
  gSystem->Load("../../neutclass/neutfsipart.so");
  gSystem->Load("../../neutclass/neutfsivert.so");
  gSystem->Load("../../neutclass/neutnucfsistep.so");
  gSystem->Load("../../neutclass/neutnucfsivert.so");
  gSystem->Load("../../neutclass/neutvect.so");
  
  TTree  *tn;
  NeutVtx *nvtx;
  NeutVect *nvect;

  TFile f("neut_minerva_numu_lowq.root");
  tn = (TTree *)(f.Get("neuttree"));

  nvtx = new NeutVtx();
  tn->SetBranchAddress("vertexbranch",&nvtx);

  nvect = new NeutVect();
  tn->SetBranchAddress("vectorbranch",&nvect);

  nevents = tn->GetEntries();

  ////////////////////////////////////////////////////////////////
  TFile *f2;
  f2 = new TFile("h_neut_minerva.root","RECREATE");

  f2->cd();

  ////////////////////////////////////////////////////////////////
  TH1D *h_Enu;
  h_Enu = new TH1D("enu","E\nu",100,0.,10.);

  ////////////////////////////////////////////////////////////////
  Double_t binsize = 0.5/50;
  Double_t weight;

  weight = 1. / binsize / normalization/25.;
  //weight = 1.;

  TH1D *h_Eavail_coarse[2];
  h_Eavail_coarse[0]= new TH1D("Eavc_low",
							   "E_Avail Coarse(0.0<q3<0.4 GeV/c)",
							   50,0.,0.5);  
  h_Eavail_coarse[1]= new TH1D("Eavc_high",
							   "E_Avail Coarse(0.4<q3<0.8 GeV/c)",
							   50,0.,0.5);  

  TH1D *h_Eavail_fine[6];
  h_Eavail_fine[0]= new TH1D("Eavc_low0",
							   "E_Avail Fine(0.0<q3<0.2 GeV/c)",
							   50,0.,0.5);  
  h_Eavail_fine[1]= new TH1D("Eavc_low1",
							   "E_Avail Fine(0.2<q3<0.3 GeV/c)",
							   50,0.,0.5);  
  h_Eavail_fine[2]= new TH1D("Eavc_low2",
							   "E_Avail Fine(0.3<q3<0.4 GeV/c)",
							   50,0.,0.5);  
  h_Eavail_fine[3]= new TH1D("Eavc_low3",
							   "E_Avail Fine(0.4<q3<0.5 GeV/c)",
							   50,0.,0.5);  
  h_Eavail_fine[4]= new TH1D("Eavc_low4",
							   "E_Avail Fine(0.5<q3<0.6 GeV/c)",
							   50,0.,0.5);  
  h_Eavail_fine[5]= new TH1D("Eavc_low5",
							   "E_Avail Fine(0.6<q3<0.8 GeV/c)",
							   50,0.,0.5);  

  ////////////////////////////////////////////////////////////////
  TH1D *h_cosmu;
  h_cosmu = new TH1D("cosmu","Cos \theta_\mu",40,-1.,1.);

  ////////////////////////////////////////////////////////////////
  TH1D *h_Q2;
  h_Q2 = new TH1D("Q2","Q^2",100,0.,10.);

  ////////////////////////////////////////////////////////////////
  TH1D *h_q3;
  h_q3 = new TH1D("q3","q3",100,0.,10.);

  ////////////////////////////////////////////////////////////////
  TH1D *h_fstat;
  h_fstat = new TH1D("fstat","fstat",21,-10.5,10.5);

  ////////////////////////////////////////////////////////////////
  Bool_t isalive;
  Int_t ip;
  Int_t mode;
  Int_t idx;

  Double_t Eavail;
  Double_t Ekine;
  Double_t xmass;

  Double_t Enu,Emu,Pmu;
  Double_t cosmu;
  Double_t xmmu;  
  Double_t Q2,q0,q3;

  for ( j = 0 ; j < nevents ; j++ ){
	tn->GetEntry(j);

	Eavail = 0;
	isalive = false;

	mode = nvect->Mode;

	// Proton and charged pions Kinetic Energy  
	/* skip initial neutrino : start from 1*/
	for ( i = 2 ; i < 4 ; i++ ){
	  if ( ((nvect->PartInfo(i))->fIsAlive) ){
		ip = (nvect->PartInfo(i))->fPID;
		if (
			( abs(ip) == 12 )||( abs(ip) == 14 ) ||
			( abs(ip) == 11 )||( abs(ip) == 13 ) ){
		  isalive = true;
		}
	  }
	}		
    
	if (isalive){
	  if ( abs(mode) != 2 ){
		idx = 2;
	  }else{
		idx = 3;
	  }

	  for ( i = idx+1 ; i < nvect->Npart() ; i++ ){
		if ( ((nvect->PartInfo(i))->fIsAlive) ){
		  h_fstat->Fill((double)((nvect->PartInfo(i))->fStatus));
		  if ( (nvect->PartInfo(i))->fStatus != 0 ){

			cout << "Intr. mode     :" << nvect->Mode   << "\n";
			
			for ( j = 0 ; j < nvect->Npart() ; j++ ){
			  cout << "j=" << j << "\n";
			  cout << "Vertex         =" << nvect->VertexID(j) << "\n";
			  cout << "Parent Index   =" << nvect->ParentIdx(j) << "\n";
			  cout << "Particle Code  = " 
				   << (nvect->PartInfo(j))->fPID   << "\n";
			  cout << "Particle Mass  = " 
				   << (nvect->PartInfo(j))->fMass   << "\n";
			  cout << "Particle Mom.  =(" 
				   << (nvect->PartInfo(j))->fP.Px() << ","
				   << (nvect->PartInfo(j))->fP.Py() << "," 
				   << (nvect->PartInfo(j))->fP.Pz() << ","
				   << (nvect->PartInfo(j))->fP.E()  << ")"
				   << "\n";
			  cout << "Particle Flag  = " 
				   << (nvect->PartInfo(j))->fIsAlive << "\n";
			  cout << "Particle Stat. = " 
				   << (nvect->PartInfo(j))->fStatus  << "\n";
			  cout << "Particle Pos(1)=(" 
				   << (nvect->PartInfo(j))->fPosIni.X() << ","
				   << (nvect->PartInfo(j))->fPosIni.Y() << "," 
				   << (nvect->PartInfo(j))->fPosIni.Z() << ","
				   << (nvect->PartInfo(j))->fPosIni.T()  << ")"
				   << "\n";
			  cout << "Particle Pos(2)=(" 
				   << (nvect->PartInfo(j))->fPosFin.Px() << ","
				   << (nvect->PartInfo(j))->fPosFin.Y() << "," 
				   << (nvect->PartInfo(j))->fPosFin.Z() << ","
				   << (nvect->PartInfo(j))->fPosFin.T()  << ")"
				   << "\n";
			}
			std::cout << "Event #" << nvtx->EventNo << "\n";
		  }

		  ip = (nvect->PartInfo(i))->fPID;
		  if ((ip == 2212)||(abs(ip) ==  211)){
			Ekine = nvect->PartInfo(i)->fP.E()-nvect->PartInfo(i)->fMass;
			Eavail += (Ekine/1000.);
		  } /* else if (( abs(ip) != 2112 )&&
					 ( abs(ip) != 12 )  &&
					 ( abs(ip) != 14 )    ){ */
		  else if (( abs(ip) == 111 )||
				   ( abs(ip) == 22  )||
				   ( abs(ip) == 12  )){ 
			Eavail += (nvect->PartInfo(i)->fP.E())/1000.0;
		  }
		}
	  }
	  
	  
	  Enu = (nvect->PartInfo(0))->fP.E()/1000.;
	  Emu  = ((nvect->PartInfo(idx))->fP.E()/1000.);
	  Pmu  = ((nvect->PartInfo(idx))->fP.P()/1000.);
	  xmmu = ((nvect->PartInfo(idx))->fMass/1000.);
	  
	  cosmu = (
			   (nvect->PartInfo(0))->fP.Px()*(nvect->PartInfo(idx))->fP.Px() +
			   (nvect->PartInfo(0))->fP.Py()*(nvect->PartInfo(idx))->fP.Py() +
			   (nvect->PartInfo(0))->fP.Pz()*(nvect->PartInfo(idx))->fP.Pz()
			   )/
		((nvect->PartInfo(0))->fP.P() * (nvect->PartInfo(idx))->fP.P());

	  q0 =(nvect->PartInfo(0))->fP.E() - (nvect->PartInfo(idx))->fP.E();
	  q0 = q0 / 1000.;
	  
	  Q2 = 2 * Enu * ( Emu - Pmu * cosmu ) - xmmu*xmmu;
	
	  q3 = sqrt(Q2 + q0*q0);

	  h_Q2->Fill(Q2);
	  h_q3->Fill(q3);
	  h_cosmu->Fill(cosmu);

	  h_Enu->Fill((nvect->PartInfo(0))->fP.E()/1000.);

	  if ( abs(mode) < 30 ){ /* select CC */
		if ( q3 < 0.2 ) {
		  h_Eavail_fine[0]->Fill(Eavail,weight/0.2);
		  h_Eavail_coarse[0]->Fill(Eavail,weight/0.4);
		}else if ( q3 < 0.3 ) {
		  h_Eavail_fine[1]->Fill(Eavail,weight/0.1);
		  h_Eavail_coarse[0]->Fill(Eavail,weight/0.4);
		}else if ( q3 < 0.4 ) {
		  h_Eavail_fine[2]->Fill(Eavail,weight/0.1);
		  h_Eavail_coarse[0]->Fill(Eavail,weight/0.4);
		}else if ( q3 < 0.5 ) {
		  h_Eavail_fine[3]->Fill(Eavail,weight/0.1);
		  h_Eavail_coarse[1]->Fill(Eavail,weight/0.4);
		}else if ( q3 < 0.6 ) {
		  h_Eavail_fine[4]->Fill(Eavail,weight/0.1);
		  h_Eavail_coarse[1]->Fill(Eavail,weight/0.4);
		}else if ( q3 < 0.8 ) {
		  h_Eavail_fine[5]->Fill(Eavail,weight/0.2);
		  h_Eavail_coarse[1]->Fill(Eavail,weight/0.4);
		}
	  }
	}
  }

	/***************************************************************/
	/*
	cout << "Event #        :" << nvect->EventNo << "\n";
	cout << "Target A       :" << nvect->TargetA << "\n";
	cout << "Target Z       :" << nvect->TargetA << "\n";
	cout << "VNuclIni       :" << nvect->VNuclIni << "\n";
	cout << "VNuclFin       :" << nvect->VNuclFin << "\n";
	cout << "PF Surface     :" << nvect->PFSurf   << "\n";
	cout << "PF Maximum     :" << nvect->PFMax    << "\n";
	cout << "Flux ID        :" << nvect->FluxID   << "\n";

	cout << "Intr. mode     :" << nvect->Mode   << "\n";

	for ( i = 0 ; i < nvect->Npart() ; i++ ){
	  cout << "i=" << i << "\n";
	  cout << "Vertex         =" << nvect->VertexID(i) << "\n";
	  cout << "Parent Index   =" << nvect->ParentIdx(i) << "\n";

	  cout << "Particle Code  = " << (nvect->PartInfo(i))->fPID   << "\n";
	  cout << "Particle Mass  = " << (nvect->PartInfo(i))->fMass   << "\n";
	  cout << "Particle Mom.  =(" << (nvect->PartInfo(i))->fP.Px() << ","
	  	   << (nvect->PartInfo(i))->fP.Py() << "," 
		   << (nvect->PartInfo(i))->fP.Pz() << ","
		   << (nvect->PartInfo(i))->fP.E()  << ")"
		   << "\n";
	  cout << "Particle Flag  = " << (nvect->PartInfo(i))->fIsAlive << "\n";
	  cout << "Particle Stat. = " << (nvect->PartInfo(i))->fStatus  << "\n";
	  cout << "Particle Pos(1)=(" << (nvect->PartInfo(i))->fPosIni.X() << ","
	  	   << (nvect->PartInfo(i))->fPosIni.Y() << "," 
		   << (nvect->PartInfo(i))->fPosIni.Z() << ","
		   << (nvect->PartInfo(i))->fPosIni.T()  << ")"
		   << "\n";
	  cout << "Particle Pos(2)=(" << (nvect->PartInfo(i))->fPosFin.Px() << ","
	  	   << (nvect->PartInfo(i))->fPosFin.Y() << "," 
		   << (nvect->PartInfo(i))->fPosFin.Z() << ","
		   << (nvect->PartInfo(i))->fPosFin.T()  << ")"
		   << "\n";
	}
	std::cout << "Event #" << nvtx->EventNo << "\n";

	for (i = 0 ; i < nvtx->Nvtx() ; i++){
	  cout << "i=" << i << "\n";
	  
	  cout << "Vertex Pos(1)=(" << (nvtx->Pos(i))->X() << ","
		   << (nvtx->Pos(i))->Y() << "," 
		   << (nvtx->Pos(i))->Z() << ","
		   << (nvtx->Pos(i))->T()  << ")"
		   << "\n";
	}
	*/
  f2->Write();
  exit();
}
  
  
  
