      program crscalc_minoo_ind

C     An adaptive integrator using the QUADPACK lib
C     Does the integral over phi, theta, q2, w using dqag
C     dqag uses a Gauss-Kronrod technique, see quadpack.f
C     for more info and
C     https://people.sc.fsu.edu/~jburkardt/f77_src
C                             /quadpack/quadpack.html
C
C     Author: Clarence Wret (cwret14@imperial.ac.uk,
C     clarence.wret@gmail.com)
C     Outputs dsig/dq2dwdphidcostheta

      Implicit None

#include "neutmodel.h"
#include "rscons.h"

      real enu, q2, w, costh, phi, xmlep
      real dxsec
      integer imode

      character*50 argv2

C     Get the settings from the neut card instead of hard-coding
C     XMABKGM=1.05
C     XMASPI=0.628524
C     RCA5ISPI=1.0283
      call necard

      if (iargc().ne.1 .and. iargc().ne.3 .and. iargc().ne.4 .and.
     &    iargc().ne.5) then
         print *, 'Cross-section table calculator for'
         print *, 'Minoo Kabirnezhad single pion model'
         print *, '*************************'
         print *, 'iargc is ', iargc()
         print *, 'args are either:'
         print *, 'nu type(1=e,2=m,3=t), MARES, CA50, MABKG'
         print *, 'nu type(1=e,2=m,3=t), MARES, CA50'
         print *, 'nu type(1=e,2=m,3=t)'
         print *, '*************************'
         print *, 'If unspecified, defaults are:'
         print *, 'MARES =', XMASPI
         print *, 'CA50  =', RCA5ISPI
         print *, 'MABKG =', XMABKGM
         print *, 'taken from neutcore/necard.F'
         stop
      endif

C     Vector form factor mass
      XMVSPI = 0.84

C     Set MARES
      if (iargc().eq.2 .or. iargc().eq.3 .or. iargc().eq.4) then
        call getarg(2, argv2)
        read(argv2, '(F4.2)') XMASPI
      endif
      print *, 'MARES = ', XMASPI

C     Set CA5(0)
      if (iargc().eq.2 .or. iargc().eq.3 .or. iargc().eq.4) then
        call getarg(3, argv2)
        read(argv2, '(F4.2)') RCA5ISPI
      endif
      print *, 'CA5(0) = ', RCA5ISPI

C     Set MABKG
      if (iargc().eq.4) then
        call getarg(4, argv2)
        read(argv2, '(F4.2)') XMABKGM
      endif
      print *, 'MABKG = ', XMABKGM

C     Mode
      imode = 3
      enu = 1.0
      q2 = -1.0
      w = 1.2
      costh = 0.0
      phi = 0.0
C     Set lepton to be muon
      xmlep = XMMU

      call mkdcrs(imode, enu, q2, w, 
     &            costh, phi, xmlep, dxsec)

      write(*,*) "**********************"
      write(*,*) "One integration:"
      write(*,*) "mode: ", imode
      write(*,*) "enu: ", enu
      write(*,*) "q2: ", q2
      write(*,*) "had mass:", w
      write(*,*) "theta: ", costh
      write(*,*) "phi: ", phi
      write(*,*) "xmlep: ", xmlep
      write(*,*) "xsec: ", dxsec
      
      end
