#include <iostream>

#include "NFortFns.h"

extern "C" {
  // Evaluate the differential cross-section
  FUNCTION_RETURN evdifcrs_();
  // Evaluate the pion rescattering proabbility
  FUNCTION_RETURN evpiprob_();
  // Set the Fermi Gas parameters
  void  nesetfgparams_();
  // Fill the neut model for the cross-section calculation
  void  nefillmodel_();
  // Set up the z-expansion configuration
  void  zexpconfig_();
  // Fill NEUT card and set defaults
  void necard_();
}

using namespace neut;
using namespace neut::rew;

using std::cout;
using std::endl;

NFortFns * NFortFns::fInstance = 0;
//____________________________________________________________________________
NFortFns::NFortFns()
{
  //tempbuf = std::cout.rdbuf();
}
//____________________________________________________________________________
NFortFns::~NFortFns()
{
  fInstance = 0;
}
//____________________________________________________________________________

NFortFns * NFortFns::Instance()
{
  if(fInstance == 0) {
    //cout << "NFortFns late initialization" << endl;
    static NFortFns::Cleaner cleaner;
    cleaner.DummyMethodAndSilentCompiler();
    fInstance = new NFortFns;

    // Default Parameters Used in MC Generation
    /*
       fInstance->XMAQEdef   = 1.21;
       fInstance->XMVQEdef   = 0.84;
       fInstance->KAPPdef    = 1.0;

       fInstance->XMASPIdef  = 1.21;
       fInstance->XMVSPIdef  = 0.84;

       fInstance->XMARESdef  = 1.21;
       fInstance->XMVRESdef  = 0.84;

       fInstance->XMASPIdef  = 0.95;
       fInstance->XMVSPIdef  = 0.84;
       fInstance->NEIFFdef   = 1;
       fInstance->NENRTYPEdef= 0;
       fInstance->RNECA5Idef = 1.01;
       fInstance->RNEBGSCLdef= 1.30;

       fInstance->XMANFFRESdef = 0.95;
       fInstance->XMVNFFRESdef = 0.84;
       fInstance->XMARSRESdef = 1.21;
       fInstance->XMVRSRESdef = 0.84;

       fInstance->NECOHEPIdef  = 0;
       fInstance->XMACOHdef  = 1.0;
       fInstance->RAD0NUdef  = 1.0;
       fInstance->fA1COHdef  = 0.0;
       fInstance->fb1COHdef  = 0.0;

       fInstance->NEPDFdef     = 12;
       fInstance->NEBODEKdef   = 1;

       fInstance->MDLQEAFdef   = 1;
       fInstance->MDLQEdef     = 402;

       fInstance->FEFQEdef   = 1.0;
       fInstance->FEFQEHdef  = 1.8;
       fInstance->FEFINELdef = 1.0;
       fInstance->FEFABSdef  = 1.1;
       fInstance->FEFCXdef   = 1.0;
       fInstance->FEFCXHdef  = 1.8;
       fInstance->FEFALLdef  = 1.0;
       */

    //These two kept for historical reasons
    fInstance->XMARESdef  = 1.21;
    fInstance->XMVRESdef  = 0.84;

    //All below can not be found in NEUT ROOT file at moment
    fInstance->NEPDFdef     = 12;
    fInstance->NEBODEKdef   = 1;

    fInstance->SCCFVdef = 0.0;
    fInstance->SCCFAdef = 0.0;
    fInstance->FPQEdef = 1.0;

    fInstance->SetDefaults();
  }
  return fInstance;
}
//____________________________________________________________________________

FUNCTION_RETURN NFortFns::evdifcrs()      {  return evdifcrs_(); }
FUNCTION_RETURN NFortFns::evpiprob()      {  return evpiprob_(); }
void  NFortFns::nesetfgparams() { nesetfgparams_(); }
void  NFortFns::nefillmodel()   { nefillmodel_(); }
void  NFortFns::zexpconfig() { zexpconfig(); }
void  NFortFns::necard()   { necard_(); }

//____________________________________________________________________________
void NFortFns::Reconfigure() {
  nefillmodel_();
  nesetfgparams_();
  if (nemdls_.mdlqeaf == 5) zexpconfig_();
}
//____________________________________________________________________________

void NFortFns::SetMCDefaultVal(NSyst_t syst, double val) {

  cout << endl << "NFortFns::SetMCDefaultVal(): Changing MC generated default value for systematic: " << NSyst::AsString(syst).c_str() << " from ";

  switch(syst) {

    case ( kXSecTwkDial_MaCCQEshape ) :
    case ( kXSecTwkDial_MaCCQE ) :
      cout << XMAQEdef;
      XMAQEdef = val;

      break;

    case ( kXSecTwkDial_AxlFFCCQE ) :
      cout << MDLQEAFdef;
      MDLQEAFdef = val;
      break;

    case ( kXSecTwkDial_VecFFCCQE ) :
      cout << MDLQEdef;
      MDLQEdef = val;
      break;

    case ( kXSecTwkDial_SCCVecQE ) :
      cout << SCCFVdef;
      SCCFVdef = val;
      break;

    case ( kXSecTwkDial_SCCAxlQE ) :
      cout << SCCFAdef;
      SCCFAdef = val;
      break;

    case ( kXSecTwkDial_PsFF ) :
      cout << FPQEdef;
      FPQEdef = val;
      break;

    case ( kSystNucl_CCQEPauliSupViaKF ) :
      cout << KAPPdef;
      KAPPdef = val;
      break;

    case ( kXSecTwkDial_BYOnOffDIS ) :
      cout << NEBODEKdef;
      NEBODEKdef = val;
      break;

    case ( kXSecTwkDial_FFRES ) :
      cout << NEIFFdef;
      NEIFFdef = val;
      break;

    case ( kXSecTwkDial_TypeRES ) :
      cout << NENRTYPEdef;
      NENRTYPEdef = val;
      break;

    case ( kXSecTwkDial_CA5RES ) :
      cout << RNECA5Idef;
      RNECA5Idef = val;
      break;

    case ( kXSecTwkDial_BgSclRES ) :
      cout << RNEBGSCLdef;
      RNEBGSCLdef = val;
      break;

    case ( kXSecTwkDial_MaRES ) :
      cout << XMASPIdef;
      XMASPIdef = val;
      break;
    case ( kXSecTwkDial_MvRES ) :
      cout << XMVSPIdef;
      XMVSPIdef = val;
      break;

    case ( kXSecTwkDial_MaBKGM ):
      cout << MaBKGMdef;
      MaBKGMdef = val;
      break;

    case ( kXSecTwkDial_SPPModel ):
      cout << SPPModeldef;
      SPPModeldef = val;
      break;

    case ( kXSecTwkDial_RSPiEj ):
      cout << RSPiEjdef;
      RSPiEjdef = val;
      break;

    case ( kXSecTwkDial_NECOHEPI ) :
      cout << NECOHEPIdef;
      NECOHEPIdef = val;
      break;

    default:
      cout << "NFortFns::SetMCDefaultVal() Warning: " << NSyst::AsString(syst).c_str() << " not implemented." << endl;
      return;
      break;
  }

  cout << " to " << val << endl;
}

void NFortFns::SetDefaults() {
  // Documentation: See neutmodel.h and necard.h

  // Should fill NEUT common blocks directly from NEUT standards
  // Prints some things so redirect
  //std::cout.rdbuf(NULL);
  //necard_();
  //std::cout.rdbuf(tempbuf);

  // Does not set the nework parameters

  /*
     nemdls_.sccfv = SCCFVdef;
     nemdls_.sccfa = SCCFAdef;
     nemdls_.fpqe = FPQEdef;

     nemdls_.mdlqeaf = MDLQEAFdef;
     nemdls_.mdlqe = MDLQEdef;
  // Should be 0 as the default model is 0 (Rein and Sehgal)
  neutcoh_.necohepi = NECOHEPIdef;

  nemdls_.xmaqe = XMAQEdef;
  nemdls_.xmvqe = XMVQEdef;
  nemdls_.kapp  = KAPPdef;

  //nemdls_.xmaspi = XMASPIdef;
  //nemdls_.xmvspi = XMVSPIdef;
  //nemdls_.xmares = XMARESdef;
  //nemdls_.xmvres = XMVRESdef;

  //nemdls_.xmaspi    = XMASPIdef;
  //nemdls_.xmvspi    = XMVSPIdef;
  //nemdls_.iffspi    = NEIFFdef;
  //nemdls_.nrtypespi = NENRTYPEdef;
  //nemdls_.rca5ispi  = RNECA5Idef;
  //nemdls_.rbgsclspi = RNEBGSCLdef;

  neut1pi_.xmanffres = XMANFFRESdef;
  neut1pi_.xmvnffres = XMVNFFRESdef;
  neut1pi_.xmarsres = XMARSRESdef;
  neut1pi_.xmvrsres = XMVRSRESdef;
  neut1pi_.neiff    = NEIFFdef;
  neut1pi_.nenrtype = NENRTYPEdef;
  neut1pi_.rneca5i  = RNECA5Idef;
  neut1pi_.rnebgscl = RNEBGSCLdef;

  nemdls_.xmacoh = XMACOHdef;
  nemdls_.rad0nu = RAD0NUdef;
  nemdls_.fa1coh = fA1COHdef;
  nemdls_.fb1coh = fb1COHdef;

  // Fixed parameters
  neutcard_.nefrmflg  = 0;
  neutcard_.nepauflg  = 0;
  neutcard_.nenefo16  = 0;
  neutcard_.nemodflg  = 0;

  neutcard_.nenefmodl = 1;
  neutcard_.nenefmodh = 1;
  neutcard_.nenefkinh = 1;
  neutpiabs_.neabspiemit = 1;


  //if (MCID==piscat)
  //neutcard_.nusim = 0;
  //else
  neutcard_.nusim = 1;

  // Nuclear parameter //
  // Note: Nucleus dependant so set using nesetfgparams_() after nucleus is determined
  //nenupr_.pfsurf      = 0.225;
  //nenupr_.pfmax       = 0.225;
  //nenupr_.vnuini      = -1. * (sqrt(0.9396^2 + pdfsurf^2) - 0.9396);
  //nenupr_.vnufin      =  0;
  nenupr_.iformlen    =  1;  // On by default

  // Pion FSI Tuned Parameters
  neffpr_.fefqe   = FEFQEdef;
  neffpr_.fefqeh  = FEFQEHdef;
  :A

  neffpr_.fefinel = FEFINELdef;
  neffpr_.fefabs  = FEFABSdef;
  neffpr_.fefcx   = FEFCXdef;
  neffpr_.fefcxh  = FEFCXHdef;
  neffpr_.fefall  = FEFALLdef;

  // Fixed values
  neffpr_.fefcoh =  1;
  neffpr_.fefqehf = 1;
  neffpr_.fefcxhf = 0;
  neffpr_.fefcohf = 0;
  */

  //defaults that still need to be passed through
  neutdis_.nepdf = NEPDFdef;
  neutdis_.nebodek = NEBODEKdef;

  nemdls_.sccfv = SCCFVdef;
  nemdls_.sccfa = SCCFAdef;
  nemdls_.fpqe = FPQEdef;

  neutcard_.nenefmodl = 1;
  neutcard_.nenefmodh = 1;
  neutcard_.nenefkinh = 1;
  neutpiabs_.neabspiemit = 1;

  neutcard_.nusim = 1;
}

// ***************************
// Print the FSI history
void NFortFns::print_fsihist() {
// ***************************

  std::cout << "*** FSI History Common Block *** " << std::endl;
  std::cout << std::left << std::setw(10) << "Nvert: " << fsihist_.nvert << std::endl;
  std::cout << std::left << std::setw(10) << "ivert" << std::setw(10) << "iflgvert" << std::setw(10) << "posvert[3]" << std::endl;

  for (int ivert = 0; ivert < fsihist_.nvert; ivert++) {
    std::cout << std::setw(10) << ivert << std::setw(10) << fsihist_.iflgvert[ivert];
    std::cout << "{";
    for (int j=0; j<3; j++) {
      std::cout << fsihist_.posvert[ivert][j] << ",";
    }
    std::cout << "}" << std::endl;
  }

  std::cout << std::left << std::setw(10) << "Nvcvert: " << fsihist_.nvcvert << std::endl;
  std::cout << std::left << std::setw(10) << "ip" << std::setw(10) << "ipvert" << std::setw(10) << "abspvert" << std::setw(10) << "iverti" << std::setw(10) << "ivertf" << std::setw(10) << "dirvert[3]" << std::endl;
  for (int ip=0; ip<fsihist_.nvcvert; ip++) {
    std::cout << std::setw(10) << ip << std::setw(10) << fsihist_.ipvert[ip] << std::setw(10) << fsihist_.abspvert[ip] << std::setw(10) << fsihist_.iverti[ip] << std::setw(10) << fsihist_.ivertf[ip];
    std::cout << "{";
    for (int j=0; j<3; j++) {
      std::cout << fsihist_.dirvert[ip][j] << ",";
    }
    std::cout << "}" << std::endl;
  }

  std::cout << "Fsiprob = " << fsihist_.fsiprob << std::endl;
  std::cout << "Ibound = " << posinnuc_.ibound << std::endl;

  std::cout << std::endl;
}


// ***************************
// Print the NEWORK common block (neutcore/nework.h)
void NFortFns::print_nework() {
// ***************************
  std::cout << "*** NEWORK Common Block ***" << std::endl;
  std::cout << std::left << std::setw(10) << "Mode  = " << nework_.modene << std::endl;
  std::cout << std::left << std::setw(10) << "Numne = " << nework_.numne << std::endl;
  std::cout << std::left << std::setw(10) << "ipne" << std::setw(10) << "flag" << std::setw(15) << "origin part" << std::setw(10) << "alive" << std::setw(10) << "pne[3]" << std::endl;
  for (int ip=0; ip<nework_.numne; ip++) {
    std::cout << std::left << std::setw(10) << nework_.ipne[ip] << std::setw(10) << nework_.iflgne[ip] << std::setw(15) << nework_.iorgne[ip] << std::setw(10) << nework_.icrnne[ip];
    std::cout << "{";
    for (int j=0; j<3; j++) {
      std::cout << nework_.pne[ip][j] << ",";
    }
    std::cout << "}" << std::endl;
  }
  std::cout << std::endl;
}

// ***************************
// Print the NEUTCRS common block
void NFortFns::print_neutcrs() {
// ***************************
  std::cout << "*** NEUTCRS Common Block*** " << std::endl;
  std::cout << "Crs[3] = " << neutcrscom_.crsx << " " << neutcrscom_.crsy << " " << neutcrscom_.crsz << " " << std::endl;
  std::cout << "Crsphi = " << neutcrscom_.crsphi << std::endl;
  std::cout << std::endl;
}

// ***************************
// Print the NEUTCRS common block
void NFortFns::print_allevent() {
// ***************************
  print_nework();
  print_neutcrs();
  print_fsihist();
  cout << endl << "-------------------------------" << endl;
}


// ***************************
// Print all the parameters for the models, neutcore/neutmodel.h, neutcore/necard.h
void NFortFns::print_allparams() {
// ***************************

  // Input parameters
  std::cout << std::endl << "*** Variable Model Parameters ***" << std::endl;

  // CCQE
  std::cout << "  CCQE: " << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmaqe = " <<  nemdls_.xmaqe << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmvqe = " <<  nemdls_.xmvqe << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.kapp  = " <<  nemdls_.kapp   << std::endl;
  std::cout << "------" << std::endl;

  // Coherent
  std::cout << "  Coherent: " << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmacoh = " <<         nemdls_.xmacoh   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.rad0nu = " <<         nemdls_.rad0nu   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.fa1coh = " <<         nemdls_.fa1coh   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.fb1coh = " <<         nemdls_.fb1coh   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.mdlcoh = " <<         nemdls_.mdlcoh   << std::endl;
  std::cout << "------" << std::endl;

  // 1pi mdl
  std::cout << "  Single pion: " << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmaspi = " <<      nemdls_.xmaspi   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmvspi = " <<      nemdls_.xmvspi   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmares = " <<      nemdls_.xmares   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmvres = " <<      nemdls_.xmvres   << std::endl;

  std::cout << std::left << std::setw(30) << "neut1pi_.xmanffres = " <<   neut1pi_.xmanffres << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.xmvnffres = " <<   neut1pi_.xmvnffres << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.xmarsres = " <<   neut1pi_.xmarsres << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.xmvrsres = " <<   neut1pi_.xmvrsres << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.neiff = " <<   neut1pi_.neiff << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.nenrtype = " <<   neut1pi_.nenrtype << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.rneca5i = " <<   neut1pi_.rneca5i << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.rnebgscl = " <<   neut1pi_.rnebgscl << std::endl;
  std::cout << "------" << std::endl;

  // DIS/mpi
  std::cout << "  Mpi and DIS: " << std::endl;
  std::cout << std::left << std::setw(30) << "neutdis_.nepdf = " <<       neutdis_.nepdf   << std::endl;
  std::cout << std::left << std::setw(30) << "neutdis_.nebodek = " <<     neutdis_.nebodek  << std::endl;
  std::cout << "------" << std::endl;

  // Pion rescatttering
  std::cout << "  Pion rescattering:" << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefqe = " <<     neffpr_.fefqe   << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefqeh = " <<    neffpr_.fefqeh  << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefinel = " <<   neffpr_.fefinel << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefabs = " <<    neffpr_.fefabs  << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefcx = " <<     neffpr_.fefcx   << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefcxh = " <<    neffpr_.fefcxh  << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefall = " <<    neffpr_.fefall  << std::endl;
  std::cout << "------" << std::endl;

  // Fixed parameters
  std::cout << "  Target materials:" << std::endl;

  std::cout << std::left << std::setw(30) << "neuttarget_.numbndn = " << neuttarget_.numbndn << std::endl;
  std::cout << std::left << std::setw(30) << "neuttarget_.numbndp = " <<   neuttarget_.numbndp << std::endl;
  std::cout << std::left << std::setw(30) << "neuttarget_.numfrep = " <<  neuttarget_.numfrep<< std::endl;
  std::cout << std::left << std::setw(30) << "neuttarget_.numatom = " <<  neuttarget_.numatom<< std::endl;

  std::cout << std::left << std::setw(30) << "nenupr_.pfsurf = " <<  nenupr_.pfsurf<< std::endl;
  std::cout << std::left << std::setw(30) << "nenupr_.pfmax = " <<  nenupr_.pfmax<< std::endl;
  std::cout << std::left << std::setw(30) << "nenupr_.vnuini = " << nenupr_.vnuini<< std::endl;
  std::cout << std::left << std::setw(30) << "nenupr_.vnufin = " << nenupr_.vnufin<< std::endl;
  std::cout << "------" << std::endl;


  std::cout << "  These are fixed (model) parameters [should = ()]" << std::endl;

  std::cout << std::left << std::setw(30) << "nemdls_.mdlqeaf    = " << nemdls_.mdlqeaf << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.mdlqe      = " << nemdls_.mdlqe << "(402)" << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.sccfv      = " << nemdls_.sccfv << "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.sccfa      = " << nemdls_.sccfa << "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.fpqe       = " << nemdls_.fpqe << "(1)" << std::endl;

  std::cout << std::left << std::setw(30) << "nenupr_.iformlen    = " << nenupr_.iformlen << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nefrmflg  = " << neutcard_.nefrmflg << "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nepauflg  = " << neutcard_.nepauflg << "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nenefo16  = " << neutcard_.nenefo16 << "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nemodflg  = " << neutcard_.nemodflg << "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nenefmodl = " << neutcard_.nenefmodl << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nenefmodh = " << neutcard_.nenefmodh << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nenefkinh = " << neutcard_.nenefkinh << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutpiabs_.neabspiemit = " << neutpiabs_.neabspiemit << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nusim = " << neutcard_.nusim << "(1)" << std::endl;

  std::cout << std::left << std::setw(30) << "neutcoh_.necohepi  = " << neutcoh_.necohepi << "(0)" << std::endl;

  std::cout << std::left << std::setw(30) << "neffpr_.fefcoh  = " << neffpr_.fefcoh << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefqehf = " << neffpr_.fefqehf<< "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefcxhf = " << neffpr_.fefcxhf<< "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefcohf = " << neffpr_.fefcohf<< "(0)" << std::endl;

  std::cout << std::endl;
}


