// NEUT ReWeight
#include <iostream>
#include "NReWeight.h"
#include "neutvect.h"
#include "NReWeightCasc.h"
#include "NReWeightNuXSecCCQE.h"
#include "NReWeightNuXSecCCRES.h"
#include "NReWeightNuXSecCOH.h"
#include "NReWeightNuXSecDIS.h"
#include "NReWeightNuXSecNC.h"
#include "NReWeightNuXSecNCEL.h"
#include "NReWeightNuXSecNCRES.h"
#include "NReWeightNuXSecRES.h"
#include "NReWeightNuclPiless.h"
#include "NSyst.h"
#include "NSystUncertainty.h"

// Common block variables for NEUT
#include "nefillverC.h"
#include "necardC.h"
#include "neutmodelC.h"
#include "neutparamsC.h"
#include "neworkC.h"
#include "fsihistC.h"
#include "neutcrsC.h"

// NEUT class headers
#include "neutvect.h"
#include "neutpart.h"
#include "neutfsipart.h"
#include "neutfsivert.h"
#include "neutrootTreeSingleton.h"

int main(int argc, char *argv[]) {

  //std::vector<neut::rew::NSyst_t> DialVector;
  //std::vector<std::vector<double> > ReWeightVals;
  if (argc != 2) {
    std::cerr << "Syntax is: ./tester OUTPUT_NEUT_ROOT_FILE.root" << std::endl;
    return 1;
  }

  std::string InputName = std::string(argv[1]);

  // List of weight engines
  //
  // NReWeightNuXSecCasc
  // NReWeightNuXSecINuke
  //
  // NReWeightNuXSecCCQE
  // NReWeightNuXSecCCRES
  // NReWeightNuXSecCCCOH
  // NReWeightNuXSecCCDIS
  //
  // NReWeightNuXSecNC
  // NReWeightNuXSecNCEL
  // NReWeightNuXSecNCRES
  // NReWeightNuXSecRES
  // NReWeightNuXSecPiless

  neut::rew::NReWeight rw;
  rw.AdoptWghtCalc("CC1pi", new neut::rew::NReWeightNuXSecCCRES);
  //rw.AdoptWghtCalc("CCQE", new neut::rew::NReWeightNuXSecCCQE);

  //std::string DialName =
  neut::rew::NSyst_t Systematic = kXSecTwkDial_CA5CCRES;
  //neut::rew::NSyst_t Systematic = kXSecTwkDial_MaCCQE;

  //kXSecTwkDial_MaBKGM;
  //kXSecTwkDial_RSPiEj;
  //kXSecTwkDial_SPPModel;

  // Initialise the systematic
  rw.Systematics().Init(Systematic);


  // Open a file
  TFile * Infile = new TFile(InputName.c_str(), "OPEN");
  if(!Infile){
    std::cerr << "Cannot open neutroot file!" << std::endl;
    throw;
  }

  // Open a TTree
  TTree * Tree = (TTree*) Infile->Get("neuttree");
  if(!Tree){
    std::cerr << "Cannot find neuttree!" << std::endl;
    throw;
  }

  NeutVect *nvect = NULL;

  Tree->SetBranchAddress("vectorbranch", &nvect);
  Tree->GetEntry(0);

  int nEvents = Tree->GetEntries();
  int PrintWidth = nEvents/20;

  // Number of reweights
  const int NReWeights = 5;
  double ReWeightVals[NReWeights] = {-1.0, -0.5, 0.0, 0.5, 1.0};

  // Loop over the reweights
  for (int i = 0; i < NReWeights; ++i) {

    // Set the systematics
    rw.Systematics().Set(Systematic, ReWeightVals[i]);
    // Reconfigure the reweight engine
    rw.Reconfigure();

    // And each event
    for (int j = 0; j < nEvents; ++j) {

      if (j % PrintWidth == 0) {
        std::cout << "On event " << j << "/" << nEvents << " (" << int(double(j)/double(nEvents)*100.0) << "%)" << std::endl;
      }

      Tree->GetEntry(j);
      // Need to fill nework first
      rw.FillNeutCommons(nvect);
      //rw.PrintNeutAll();
      double weight = rw.CalcWeight();
      if (weight != 1.0) {
        std::cout << "*******" << std::endl;
        std::cout << weight << std::endl;
      }
    }
  }

  return 0;
}
